import numpy as np
import os

#code to call fortran to calculate the response function


dirfort = '/Users/ds207/Documents/standrews/sta/fort/fortcode'
pwd = os.getcwd()

iexist = os.path.exists('tfb_pycall.exe')


def mytfb_pycall(tau,wav,uembh,uemdot,deginc,slopevisc=-0.75,slopeirad=-0.75,urin=3.0,r0=1.0,eta=0.1,uhx=3.0,forcetau0 = 1,tfx=0,T0v=1.e4,T0x=1.e4,sv=-0.75,sx=-0.75,newversion = 1):
 

 taulo = tau[0]
 tauhi = tau[-1]
 dtau = np.mean(tau[1:] - tau[:-1])

 if (tfx == 0):

  if (iexist == 0 or newversion == 1):
   print('compiling...')
   dirpycallf90 = '/Users/ds207/Documents/standrews/sta/fort/fortcode/delaydist/tfb_pycall.f90'
   os.system('cp '+dirpycallf90+' '+pwd)
   os.system('gfortran tfb_pycall.f90 '+dirfort+'/delaydist/tfb_redshifttest.f90 '+dirfort+'/delaydist/tfb_x.f90 '+dirfort+'/cgs.f90'+' '+dirfort+'/itp2.for '+dirfort+'/ran_new2.f90 -o tfb_pycall.exe')
   newversion = 0
   print('done compiling')

  f = open('tfb_pycall.par','w')
  f.write(np.str(wav)+'\n')
  f.write(np.str(taulo)+' '+np.str(tauhi)+' '+np.str(dtau)+'\n')
  f.write(np.str(urin) + '\n')
  f.write(np.str(uembh) + '\n')
  #f.write(np.str(uemdot) + '\n')
  f.write(np.str(deginc) + '\n')
  f.write(np.str(uhx) + '\n')
  f.write(np.str(eta) + '\n')
  f.write(np.str(-1*slopevisc)+'\n')
  f.write(np.str(-1*slopeirad)+'\n')
  f.close() 
  os.system('./tfb_pycall.exe')
  dat = np.loadtxt('tfb_pycall.op')
 
 else:

  
  if (iexist == 0 or newversion == 1):
   print('tfx mode is on!!!')
   print('compiling...')
   dirpycallf90 = '/Users/ds207/Documents/standrews/sta/fort/fortcode/delaydist/tfx_pycall.f90'
   os.system('cp '+dirpycallf90+' '+pwd)
   os.system('gfortran tfx_pycall.f90 '+dirfort+'/delaydist/tfb_x.f90 '+dirfort+'/myrs2ld.f90 '+dirfort+'/ran_new2.f90 '+dirfort+'/mytr.f90 -o tfx_pycall.exe')
   print('done compiling')
   
  f = open('tfx_pycall.par','w')
  f.write(np.str(wav)+'\n')
  f.write(np.str(taulo)+' '+np.str(tauhi)+' '+np.str(dtau)+'\n')
  f.write(np.str(urin) + '\n')
  f.write(np.str(uembh) + '\n')
  f.write(np.str(deginc) + '\n')
  f.write(np.str(T0v) + '\n')
  f.write(np.str(T0x) + '\n')
  f.write(np.str(-1*sv) + '\n')
  f.write(np.str(-1*sx) + '\n')
  f.close()
  os.system('./tfx_pycall.exe')
  dat = np.loadtxt('tfx_pycall.op')
  
 if (forcetau0 == 1):
  dat[0,1] = 0
 
 return(dat)
 
 


import numpy as np
#t,x,sig,f,np.array(ampspec),np.array(sigampspec),np.array(phasespec),np.array(sigphasespec)
# outputs t,x,sig data and errors
# ;       f, ampspec,sigampspec,phasespec,sigphasespec # frequencies, amplitude spectrum and errors and phase

proprtdtmin = 0.02

def mypowerspec(fname):    
    

    dat =np.loadtxt(fname)
    t   =dat[:,0]
    thi =t.max()
    tlo = t.min()
    tlen = thi - tlo
    x   =dat[:,1]
    sig =dat[:,2]
#    sig = sig*0 + 0.1
    nx  = x.shape[0]
    nxmod = nx*10
    dtmod = (t[-1]-t[0])/(nxmod-1)
    tmod=np.arange(t[0],t[-1]+dtmod, dtmod)
    nxmod=tmod.shape[0]
    
    xop = np.zeros(nxmod)
    sop=[]
    sigsop=[]
    cop=[]
    sigcop=[]
    ampspec=[]
    sigampspec=[]
    phasespec=[]
    sigphasespec=[]
    
#specify the lower and upper frequency limits
    dt=t[1:] - t[:-1]
    dt=dt[np.nonzero(dt)] #problem fixed if 2 datapoints taken at same time
    dtmin = dt.min() #this is the interval between points

# this fixes data points that are super close together making the nyquist frequency higher than can be computed in a reasonable time.    
    if (dtmin < properdtmin):
        print 'Time resolution too high, using average time resolution...'
        dtmin = (tlen/(nx-1))
        if (dtmin < properdtmin):
            print 'The time resolution on this lightcurve is better than sense! I dont like it and will use',properdtmin,'instead.'
            dtmin = properdtmin
    


    nf  = np.int( np.ceil(0.5*nx) )
    fhi = 0.5/dtmin              
    flo = 1/(t[-1]-t[0])
    df = 0.5*(fhi - flo)/(nf-1)
    f = np.arange(flo,fhi+df,df)
    nf = f.shape[0]
    
    w = 2*np.pi*f
# subtract the mean from the data (to avoid worry about the constant offset
    xmean = np.median(x)
    xinp = x - xmean
    xop = np.ones(nxmod)*xmean
    
    
    for i_f in range(nf):
        optscalop = myoptscal(t,xinp,sig,f[i_f])
        s    = optscalop[0]
        sigs = optscalop[1]
        c    = optscalop[2]
        sigc = optscalop[3]
        sop.append(s)
        sigsop.append(sigs)
        cop.append(c)
        sigcop.append(sigc)
        

             
#these bits are updated every i_f
        fit = s*np.sin(w[i_f]*t) + c*np.cos(w[i_f]*t)
        xinp = xinp - fit
        fitmod = s*np.sin(w[i_f]*tmod) + c*np.cos(w[i_f]*tmod)
        
#        print fitmod.mean(),i_f,'test',np.std(fitmod)
# if you fit to too high frequencies, the model goes to crazy high values, introduce a check for this below.
        if (np.abs( np.std(fitmod) ) > 100*xmean):
            print 'The fit has gone crazy! you have tried to fit to too high a frequency... stopping here.'
            nf = len(ampspec)
            break

# amplitude spectrum        
        sigssq = 2*s*sigs
        sigcsq = 2*c*sigc
        a =s*s+c*c
        siga = np.sqrt(sigssq*sigssq + sigcsq*sigcsq)
        ampspec.append(np.sqrt(a))
        sigampspec.append( 0.5/np.sqrt(a)* siga )

# phase spectrum
        soverc    = s/c
        sigsoverc = soverc * np.sqrt( (sigs/s)**2 + (sigc/c)**2 )
        phasespec.append( np.arctan(-1.*soverc) )
        sigphasespec.append( np.abs( sigsoverc/(1. + soverc*soverc) ) ) 

# update the plot model
        xop = xop + fitmod
        


# fit a broken power law to the amplitude spectrum
    ampspeclog = np.log10(np.array(ampspec))
    sigampspeclog = np.abs( np.array(sigampspec)/(np.array(ampspec)*loge10 ) )
   
    return(t,x,sig,f,np.array(ampspec),np.array(sigampspec),np.array(phasespec),np.array(sigphasespec)) 


import scipy.interpolate as si
import numpy as np
import scipy

#inputs
#x,y,z data
#xi,yi desired interpolation grid

def my2ditp(x,y,z,xi,yi,gauss=0):
 
 #do standard stuff
 xim, yim = np.meshgrid(xi, yi)
 rbf = scipy.interpolate.Rbf(x, y, z, function='linear')
 zim = rbf(xim, yim)
 
 nia = np.shape(zim)[0]

 
 
 ri = xi**2 + yi**2
 r  = x**2 + y**2
 #now alter zi so that it is treated differently when data is extrapolated
 for ia in range(nia):
  
  xinow = xim[:,ia]
  yinow = yim[:,ia]
  rinow = np.sqrt(yinow**2 + xinow**2)
  
  xinow0 = xinow[0]
  if (xinow0 > 0):
   try:
    idx = np.where(x > xinow0)[0]
    ymax = np.max(y[idx])
    idxim = np.where(yinow > ymax)[0]
    zim[idxim,ia] = 0
   except:
    pass  
  elif (xinow0 < 0):
   try:
    idx = np.where(x < xinow0)[0]
    #print idx
    ymax = np.max(y[idx])
    idxim = np.where(yinow > ymax)[0]
    zim[idxim,ia] = 0
   except:
    pass
  
  #print xinow0
  #print yinow
  #print zim[:,ia]
  #raw_input()
 return(zim)
   


## Python script to calculate the mean time delay for a face on disk
## this is not the mean delay. Just the delay where x=hc/(lamkT) =1
import numpy as np

G=6.673e-11
h=6.626e-34
k=1.38e-23
c=2.9979e8
pi=np.pi
msun=2e30
year=3600*24.*365

def taumeanface(lam,mmdot):
    
    x=lam*1e-10
    y=mmdot*msun*msun/year
    tdel=((45*G*y*x**4)/(16*pi**6 * c**5 *h))**(1./3)
    return(tdel)




### Script to plot an irradiated disk spectrum for a static disk

import numpy as np
import const
from pylab import *
from scipy import *


planck = 6.626307004e-34
c      = 2.99792458e8
boltz  = 1.38064852e-23
emsun  = 1.98855e30
gnewt  = 6.6740831e-11
sb     = 5.6700367e-8

####Define all the default paramters

#r_in = 3        ## ISCO radius in schwarzschild radii
#r_out = 1e5     ## desired outer radius of accretion disk 
#delta_r = 0.1  ## desired radial resolution 
#M = 1e8         ## Black hole mass (solar masses)
#D = 1 			## redshift distance to black hole
#inclination = 0 ## desired inclination (radians)
#M_dot = 1 		## Accretion rate (solar masses per year)

## Convert to SI units
#M = M*emsun
#rs = 3000*M/emsun
#r_in = r_in*rs
#r_out = r_out*rs
#delta_r = delta_r*rs
#M_dot=M_dot * const.Msun





##define the temperature radius profile for a steady state accretion disk in parsecs
##viscous dissipation due gives rise to a factor of 3/8pi
def T_R(M,M_dot,R,R_in):
	T=(3*gnewt*M*M_dot*(1-(R_in/R)**0.5)/(8*np.pi*sb*R**3))**0.25
	return(T)




###############################################################################################
##make the following code a definition
def flat_disk(r_out=1e5,r_in=1,inc=0,D=75.,embh=1.e7,emdot=1.0):



	####Define all the default paramters
	if default=='d':
		r_in = 3        ## ISCO radius in schwarzschild radii
		r_out = 1e5     ## desired outer radius of accretion disk 
		delta_r = 0.1  ## desired radial resolution 
		M = 1e8         ## Black hole mass (solar masses)
		D = 1 			## redshift distance to black hole
		inclination = 0 ## desired inclination (radians)
		M_dot = 1 		## Accretion rate (solar masses per year)
		                ## Total luminosity of point x ray source (see frank, king , rainne)

	## Convert to SI units
	M = embh*emsun
	rs = 3000*embh
	r_in = r_in*rs
	r_out = r_out*rs
	delta_r = delta_r*rs
	M_dot=emdot * emsun

	
	### Luminosity of the central source (powered by acrretion. May need to introduce efficiency
	##parameter later
	Lum = gnewt * M * M_dot / r_in
	
	##populate the x and y dimensions of the array with radial and azimuthal coordinates 
	rad_inf=1.*np.arange(np.ceil((r_out-r_in)/delta_r))*(r_out-r_in) + r_in
	

	

	### Obtain the temperature radius relation, wavelengths of the corresponding peak wavelengths
	## and solid angle element of each radii
	s_a=2*np.pi*rad_inf*delta_r*np.cos(inc)/D**2
	T=T_R(M,M_dot,rad_inf,r_in)	
	wavelength = const.w_c/T


	##plank spectrum as a function of wavelength have to sum over temperature contributions from all of disk
	B_T=np.zeros(len(rad_inf))
	for i in range(len(rad_inf)):
		B_T[i]=(2*const.plank*const.c**2/wavelength[i]**3 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T)) - 1)**-1 * s_a).sum()


	
	
	
	####Now define the emission associated by irradiation from point source
	## A thick disc has a depth (H) as a function of radius (x) given by 
	alph=9./7
	
	h_inf=rad_inf**9./7
	diff_h=9./7*rad_inf**2/7
	
	
	##solid angle element
	##see note book for derivation
	sa2=2*np.pi*rad_inf/D**2*(np.sin(inc)-np.cos(inc)**diff_H)
	
	##Also different temperature profile (see note book or frank, king and rainne chapt 5.10)
	
	T_r=(Lum/(4*np.pi*rad_inf**2*const.sb)*h_inf/rad_inf*(1.*alpha-1.)*(1-beta))**0.25
	
	## flux. As with plank spectrum
	for i in range(len(rad_inf)):
		B_T_r[i]=(2*const.plank*const.c**2/wavelength[i]**3 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T_r)) - 1)**-1 * sa2).sum()
	
	
	
	
	
	
	
	## Just to plot everything

	output=0
	if output == 1:
		plot(math.log(wavelength,10),math.log(B_T,10))
		xlabel('Wavelength / m')
		ylabel('flux Wm^-1')
		xticks(fontsize=15)
		yticks(fontsize=15)
		title ('Z = '+str(text[0])+' .  r_in/r_out (pc)=' +str(text[1])+'/'+str(text[2])+'.  M='+str(text[4])+'.  M dot='+str(text[3])+'.  Inc='+str(text[5]))
		show()
	
	return(wavelength,B_T,B_T_r)



#generate a light curve with a bbroken power law power spectrum according to the prescription of timmer and kronig 1995




import numpy as np
#import myrandom as mr
import matplotlib.pylab as plt



def mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=100,dt=0.125,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0):
 '''
 Generate random walk light curve. Can also customize the powr spectrum slope using a and b arguments
 :param datfile:
 :param p0:
 :param f0:
 :param a:
 :param b:
 :param tlo:
 :param thi:
 :param dt:
 :param ploton:
 :param iseed:
 :param meannorm:
 :param sdnorm:
 :return:
 '''
 #!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!
 #Input parameters
 
 #use clock to get different random light curve each call if -1, else iseed specifies light curve
 if (iseed > 0):
  np.random.seed(int(iseed))
 else:
  np.random.seed()
 
 flo = 0.5/(thi-tlo)
 fhi = 1./dt
 #datfile ='mylcggen_op.dat'
 #
 #p0 = 1.0      # scale factor 
 #f0 = 0.1      #break frequency cycles per day
 #a  = -2.0
 #b  = -2.0     # second and first slopes of the broken power law
 #tlo = 0.0
 #thi = 100.0
 #dt  = 0.1
 #
 #flo = 0.01
 #fhi = 4.0     # low and hi fourier frequency limits 
 df = 1.*flo
 #
 #ploton = 1
 
 
 #!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!
 
 
 
 
 
 
 
 
 
 
 
 
 
 #!!!!!!!!!#!!!!!!!!!#!!! Make and save the light curve !!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!
 
 time = np.arange(tlo,thi+dt,dt)
 nt = np.shape(time)[0]
 x    = np.zeros(nt)
 dat  = np.zeros((nt,2))
 
 nf = int(np.ceil((fhi - flo)/df + 1))
 freq = np.arange(flo,fhi+df,df)
 w    = 2*np.pi*freq
 
 a   = np.sqrt( p0*(freq/f0)**a / (1 + (freq/f0))**(a-b))
 sk  = np.random.randn(nf)*a#mr.normdis(nf,0,1)*a
 ck  = np.random.randn(nf)*a#mr.normdis(nf,0,1)*a
 
 dps = np.zeros((nf,3))
 dps[:,0] = freq
 dps[:,1] = sk
 dps[:,2] = ck
 
 #print 'mylcgen iseed...',iseed
 for i in range(nt):
  tnow = time[i]
  x[i] = np.sum(sk*np.sin(w*tnow) + ck*np.cos(w*tnow))
  #print nt, i,'here'
 dat[:,0] = time
 dat[:,1] = x
 
 if (len(datfile) > 0):
  np.savetxt(datfile,dat)
  np.savetxt('fft_'+datfile,dps)
 #!!!!!!!!!#!!!!!!!!!#!!!#!!!!!!!!!#!!!!!!!!!#!!!#!!!!!!!!!#!!!!!!!!!#!!!#!!!!!!!!!#!!!!!!!!!#!!!
 
 
 
 #normalise if option turned on
 if (sdnorm > 0):
  datsd = np.std(dat[:,1])
  datmean = np.mean(dat[:,1])
  dat[:,1] = (dat[:,1] - datmean)*sdnorm/datsd + datmean
 
 if (meannorm >= 0):
  dat[:,1] = dat[:,1] - dat[:,1].mean() + meannorm
 
 
 #!!!!!!!!!#!!!!!!!!!#!!! If ploton make power spectrum and plot#!!!!!!!!!#!!!!!!!!!#!!!#!!!!!!!!!#!!!!!!!!!#!!!
 if (ploton == 1):
  fig = plt.figure()
  ax1=fig.add_subplot(211)
  ax1.plot(time,x)
  
  ax2=fig.add_subplot(212)
  ax2.plot(freq,sk**2+ck**2,ls='',marker='o')
  ax2.set_xscale('log')
  ax2.set_yscale('log')
  plt.show()
 
 return(dat)








#python  code to give me the range of a set of data
import numpy as np
import os

pwd=os.getcwd()

dir='../fort/fortcode/mcmcmulti3/5548ltonly_july28/'
os.chdir(dir)

		
	
			
	
fname=['converted_u_mjd.dat','converted_g_mjd.dat','converted_r_mjd.dat','converted_i_mjd.dat','converted_z_mjd.dat']#['resamp_fake_3556.51_nonoise.dat','resamp_fake_4702.48_nonoise.dat','resamp_fake_6175.57_nonoise.dat','rebinned_fake_6300.00_nonoise.dat','resamp_fake_7489.97_nonoise.dat','resamp_fake_8946.70_nonoise.dat']
nfile=len(fname)
fracvar=np.zeros(nfile)
mean=np.zeros(nfile)
sd=np.zeros(nfile)


for i in range (nfile):
    dat=np.loadtxt(fname[i])
    sd[i]=np.std(dat[:,1])
    mean[i]=np.mean(dat[:,1])
    fracvar[i]=sd[i]**2/mean[i]**2
    
print 'the mean s...'
print mean
print 'the sd s'
print sd
print 'the fracvar s...'
print fracvar
print 'the end'

os.chdir(pwd)


import numpy as np
from sklearn import linear_model
from sklearn.metrics import mean_squared_error, r2_score
import matplotlib.pylab as plt

#function to identify the contribution of constituent signals to a prent signal

def constituents_sklearn(y_main,y_parts):

 reg = linear_model.LinearRegression()
 reg.fit(y_parts,y_main)                                      
 linear_model.LinearRegression(copy_X=True, fit_intercept=True, n_jobs=None,
                 normalize=False)

 coefficients = reg.coef_
 
 
 #split into train and test
 ndat,ndim = np.shape(y_parts)
 frac = 0.85
 ntest = np.int(frac*ndat)
 yt_main = y_main[:ntest]
 yt_parts = y_parts[:ntest,:]
 # Create linear regression object
 reg_test = linear_model.LinearRegression()
 # Train the model using the training sets
 reg_test.fit(yt_parts, yt_main)
 # Make predictions using the testing set
 ytp_main = regr.predict(yt_parts[ntest:,:])
 mse = mean_squared_error(y_main[ntest:],ytp_main)
 # Explained variance score
 yp_main = reg.predict(y_parts)
 r2s = r2_score(y_main,yp_main)

 
 return(coefficients,mse,r2s)
 
 
 
 
 
 
#custom code to fit the glm to multivariate time series components
#y_main(npoints): numpy array of main time series (flows)
#y_parts(npoints,ndimension): 2d numpy array of points for each proposed constituent time series
def constituents_fit(y_main,y_parts,trainfrac = 0.8):
 
 ny = len(y_main)
 ny,npatterns = np.shape(y_parts)
 
 #normalise
 yp_mean = 0#np.mean(y_parts,axis=0)
 yp_std  = 1#np.std(y_parts,axis=0)
 ym_mean = 0#np.mean(y_main)
 ym_std  = 1#np.std(y_main)
 yn_main = (y_main - ym_mean)/ym_std
 yn_parts = (y_parts - yp_mean)/yp_std
  
 
 #compute hes and covariance
 def fit(x,y):
  hnow  = np.tensordot(x.T,x,axes=1)
  cnow  = np.dot(y,x)
  cov   = np.linalg.inv(hnow)
  parms = cov.dot(cnow)
  return(parms,cov)
 parms,cov = fit(yn_parts,yn_main)
 y_mod = np.sum(parms*yn_parts,axis=1)
 
 #compute the r2 score describes how much of the variance the model explains
 var = np.std(y_main)
 var_mod = np.std(y_main-y_mod)
 r2 = 1. - var_mod/var
 
 
 #train the model on a fraction of the input data to compute the model mse
 idtrain = np.int(frac*ny)
 parms_train, cov_train = fit(yn_parts[:idtrain,:],yn_main[:idtrain])
 y_mod_predict = np.sum(parms_train*yn_parts[idtrain:,:],axis=1)
 mse = np.sum((y_mod_predict - yn_main[idtrain:])**2)/(ny-idtrain)
 
 #importance will be the contribution of each amplitude to the model
 #(a measure of the contribution of each component to the explained variance)
 importance = parms**2/np.sum(parms**2)
 
 return(parms,cov,r2,mse,importance)
 
 
 
 
 
def constituents_predict(parms,x):
 predict = np.sum(parms*x,axis=1)
 return(predict) 
 
 
 
 



def constituents_diagplot(importance,labels=None,fig_title='show'):
 ncompare = len(importance)
 color = ['r','g','b','k','purple','cyan','orange','yellow','skyblue'] 
 fig = plt.figure()
 ax1 = fig.add_subplot(111)
 xbar = np.arange(ncompare)+1
 barwidth = 0.8
 
 for i in range(ncompare):
  npar = len(importance[i])
  c = color*npar
  if (labels == None): 
   lab_ann = ['parameter '+np.str(i2+1) for i2 in range(npar)]
   y = np.cumsum(importance[i])
   x = [xbar[i]]*npar
   ax1.bar(x,y[::-1],width=barwidth,color=c[:npar],label=None)
   for i2 in range(npar):
    if i2 == 0:
     yannotate = y[0]/2
    else:
     yannotate = (y[i2] - y[i2-1])/2 + y[i2-1]
    ax1.annotate(lab_ann[i2],(xbar[i]-barwidth/2,yannotate))
 ax1.set_ylabel('explained variance fraction')
 ax1.set_xlabel('model number')
 ax1.set_xticks(xbar)
 if fig_title == 'show':
  plt.show()
 else:
  plt.savefig(fig_title)
 return()
 
 
 

#test on fake data 
x = np.arange(0,100,0.1)
y = 2.3*x + 4.*x**2 + 0.3*x**3#np.sin(2*np.pi*x/30)
y = y
ypart = [x,x**2,x**3]
ypart = np.array(ypart).T
ny,ndim = np.shape(ypart)


#test sklearn glm code
reg = linear_model.LinearRegression()
reg.fit(ypart,y)                                      
a=linear_model.LinearRegression(copy_X=True, fit_intercept=True, n_jobs=None,
                normalize=True)



#test custom code
frac = 0.9
ntest = np.int(frac*ny)
ytrain = y[:ntest]
ytest = y[ntest:]
ypart_train = ypart[:ntest,:]
ypart_test  = ypart[ntest:,:]
parms,cov,r2,mse,importance = constituents_fit(ytrain,ypart_train,trainfrac = 0.8)
ypredict = constituents_predict(parms,ypart_test)





#diagnostic figure
fig = plt.figure()
ax1 = fig.add_subplot(211)
ax1.scatter(x[:],y[:],color='b',label='Data')
ax1.scatter(x[ntest:],ypredict,color='r',label='prediction')

ax2=fig.add_subplot(212)
xbar = np.arange(ndim)
ax2.bar(xbar,importance*r2,label='explained variance')
ax2.set_xlabel('parameter')
ax2.set_ylabel('explained variance')

plt.show()


constituents_diagplot([importance]*3)









import numpy as np

pi=np.pi
e=np.e
##This line generates n random numbers between a and b distributed uniformly

def unirand(no,a,b):
	dat=1.*(b-a)*np.random.rand(no)+1.*a
	return(dat)

	
###This definition is udentical to the previous but returns samples from a Gaussian distribution
def unirand_n(no,sigma,mu,a,b):
	dat_n=1./(sigma**2*2*pi)**0.5 * e**(-(unirand(no,a,b)-mu)**2/(2*sigma**2))
###	dat_n=np.random.randn(no)
	return(dat_n)
	

	
def unirand_e(no,tau,a,b):
	#dat_e=1./tau*e**(-np.random.rand(10**4)/tau)
	dat_e=1./tau*e**(-unirand(no,a,b)/tau)
	return(dat_e)
	
	
	

def normdis(n,mu,sigma):
#draw n random numbers from gaussian distribution with mean mu and sd sigma
 op = np.random.randn(n)*sigma + mu
 return(op)	
	
def normdisoldandbroken(n,mu,sigma):	
# for some reason, must set n to one otherwise get odd distribution of random numbers 24/10/2014

#n=1000000
#sigma=1
#mu=0
## draw n random numbers uniformly
	view_out = []
	count=0
	while count < n:
		randval=unirand(n, mu-10*sigma, mu+10*sigma)
		randval.sort()

		randprob=np.zeros((n,2))
		randprob[:,0]=1./(sigma**2*2*pi)**0.5 * e**(-(randval-mu)**2/(2*sigma**2))
		randprob[:,0]=randprob[:,0]/np.max(randprob[:,0])     ##the probablility that the chosen data point will remain
		randprob[:,1]=unirand(n,0,1)

#remove all data points where the random number just drawn between 0 a d 1 is less than the gausian prob for each f the original random numbers
		view=np.delete(randval,[np.where(randprob[:,0] < randprob[:,1])])
	
		view_out=np.append(view_out,view,axis=1)
		count = len(view_out)
		#print 'generating gaussian... ' + str(count) + ' of ' + str(n)
	## delete elements in the view_out array untill we have the correct number of points
	#diff= len(view) - n
	view_out=view_out[:n]
	return(view_out)
#normdis=np.vectorize(normdis)	



import numpy as np
from mylcgen import *
from myrandom import *
import matplotlib.pylab as plt
import mysinecostrans as msc

#generate test light curve and add noise
dat      = mylcgen(tlo=0,thi=100,dt=2.0,iseed=42342332)
ndat     = np.shape(dat[:,0])[0]
datmean  = np.std(dat[:,1])
sig      = np.ones(ndat)/10*datmean
for i in range(ndat):
 dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
y        = dat[:,1]
time     = dat[:,0]


tlo = np.min(time)
thi = np.max(time)
dt  = np.mean(time[1:]-time[:-1])

flo = 0.5/(thi-tlo)
fhi = 5./dt

msc.sct(time,y,sig,freq=np.arange(flo,fhi+flo,flo),costrans = 0)


#test the routine above and do it manually to compare

tlo = np.min(time)
thi = np.max(time)
dt  = np.mean(time[1:]-time[:-1])




#if cosine tranform, use only coses, else if sine transform use only sines
costrans = 0




if (costrans == 1):
 phase = 0
elif (costrans == 0):
 phase = -np.pi/2


#input some minimum and maximum frequency range for the Fourier terms
flo = 0.5/(thi-tlo)
fhi = 5./dt

#angular frequency
w = np.arange(flo,fhi+flo,flo)*2*np.pi

#now compute the hessian matrix
nw = np.shape(w)[0]
np2 = 2*nw

cvec = np.ones(nw)
hes = np.ones((nw,nw))
sig2 = sig*sig
#save the sine and cosine value in  2d array
cwt = np.ones((ndat,nw))
swt = np.ones((ndat,nw))
for iw in range(nw):
 cwt[:,iw] = np.cos(w[iw]*time +phase)

for ikp in range(nw):
 wp = w[ikp]
 
 wp2 = wp*wp
 cvec[ikp]   = np.sum(y*cwt[:,ikp]/sig2)
 #print ikp,nw
 for ik in range(nw):
  #cosine column
  hnow_c = np.sum(cwt[:,ik]*cwt[:,ikp]/sig2)
  if (ik == ikp):
   hnow_c = hnow_c + wp2
  hes[ikp,ik]   = hnow_c

   

      
#invert the matrix to get the covariance matrix
cov = np.linalg.inv(hes)


#find the parameters
parm  = cov.dot(cvec)
ckout = parm









#plot the input and output light curves to see if they match
freq = w/2/np.pi
dtres = dt/10
tplot = np.arange(tlo,thi+dtres,dtres)
xplot = []
for tnow in tplot:
 xplot.append( np.sum(ckout*np.cos(w*tnow + phase) ) )

fig = plt.figure()
ax1 = fig.add_subplot(211) 
ax1.errorbar(time,y,sig,ls='',color='k')
ax1.plot(tplot,xplot,color='b')
ax1.set_xlabel('Time (days)')
ax1.set_ylabel('flux')


ax2 = fig.add_subplot(212) 
ax2.plot(freq,ckout**2,ls='',marker='o')
ax2.set_xscale('log')
ax2.set_yscale('log')
ax2.set_xlabel('Frequency (cyc/day)')
ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
plt.savefig('test_r_hesfit_cosine.pdf')


#
#
#
#
#
#
#


    


import numpy as np
import myreadlines

#define an array to load in parameters and reject certain values, returning the masked
#points in an array idxmask

def myloadpar(fname,rejectinf==1,rejectnan==1,rejectother==[0]):
 try:
  dat = np.loadtxt(fname)
 except:
  dat = myreadlines.myreadlines(fname)

 nd  = dat.shape
 nalong = nd[1]
 nvert = nd[0]
 idxmask = np.zeros(nvert)
 nother = len(rejectother)
 
 for idx in range(nalong):
  x = np.median(dat[:,idx])
  
  if (rejectnan == 1):
   idx_xnan = np.isnan(abs(x))
   idxmask[idx_xnan] = 1

  if (rejectinf == 1):
   idx_xinf= abs(x) == np.inf
   idxmask[idx_xinf] = 1

  for i2 in range(nother):
   val = rejectother[i2]
   idx_num = x == val
   idxmask[idx_num] = 1
   
 datnew = dat[idxmask==0,:]
 return(datnew,idxmask)
   



#function to annotate a polygon id column in a dataframe with the polygon name

import sys
sys.path.insert(0, '/Users/david/github_datascience/vortexa/analytics/utils')
import sql_utils_david as sql_utils, math_utils




def annotate_polygons(df_in,column_df,df_poly='auto',column_poly=''):
 
 if (df_poly == 'auto'):
  pd.set_option("display.max_rows", 200)
  sql_engine = sql_utils.initiate_sql_engine(DB_NAME='backend_nov7')
  poly = pd.read_sql("""select * from new_polygons""")
 else:
  poly = df_poly
  
  
  
 if (column_poly == ''):
  cp = 'id'
 else:
  cp = column_poly
 annotated = df_in.join(poly.set_index(cp),on=column_df)
 #df_in.set_index(column_df).join(poly.set_index(column_poly))
 
 annotated_accepted = annotated[(annotated[cp].notnull())
 
 print(len(annotated) - len(annotated_accepted),' entries excluded from the output due to joining failure)
 print(100.*(len(annotated) - len(annotated_accepted))/len(annotated),' pc of total entries')
 
 return(annotated_accepted)
 
 




import numpy as np
import pandas as pd








#shift time series 2 in time by 'shift' and 
#map onto lc1 by adding, multiplying by the appropriate means and rms
def shift_time(lc1,lc2,shift=0,renorm = 1,custom_mean = [],custom_rms = []):
 
 
 if (type(lc1) == pd.core.frame.DataFrame):
  x1         = lc1.values[:,1] 
  time1in    = pd.to_datetime(lc1.values[:,0]) 
  reftime    = time1in[0]
  time1      = np.array( [(t1p-reftime).days for t1p in time1in],dtype='float' )
  try:
   sig1 = lc1.values[:,2]
  except:
   print('no uncertainties given')
   sig1 = x1*0 + 1.0
 else:
  x1         = lc1[:,1]
  time1in    = lc1[:,0]
  reftime    = lc1[0,0]
  time1    = np.array(time1in - reftime,dtype='float')
  try:
   sig1 = lc1[:,2]
  except:
   print('no uncertainties given')
   sig1 = x1*0 + 1.0
 lc1_rms  = np.std(x1)
 lc1_mean = np.mean(x1) 
 
 
 
 
 #transform and interpolate second light curve onto first
 if (type(lc2) == pd.core.frame.DataFrame):
  lc2_rms  = np.std(lc2.values[:,1])
  lc2_mean = np.mean(lc2.values[:,1])
 else:
  lc2_rms  = np.std(lc2[:,1])
  lc2_mean = np.mean(lc2[:,1])
  
 #shift 2 by the input time lag 'shift' and transform onto lc1 scale
 if (type(lc2) == pd.core.frame.DataFrame):
  time2pre = pd.to_datetime(lc2.values[:,0])
  a = time2pre - reftime
  time2 = np.array([an.days for an in a],dtype='float') + shift
  x2 = lc2.values[:,1]
  try:
   sig2 = lc2.values[:,2]
  except:
   print('no uncertainties given')
   
 else:
  time2pre = lc2[:,0]
  time2 = np.array(time2pre + shift - reftime,dtype='float')
  x2    = lc2[:,1]
  try:
   sig2 = lc2.values[:,2]
  except:
   print('no uncertainties given')
  
 if (custom_mean != []):  
  x2new = np.array( (x2 - custom_mean[1])*custom_rms[0]/custom_rms[1] + custom_mean[0] ,dtype='float')
 else: 
  x2new = np.array( (x2 - lc2_mean)*lc1_rms/lc2_rms + lc1_mean ,dtype='float')
 

 
 #print(type(time1))
 #print(type(time2))
 #print(type(x2new))
 #print(np.mean(x2new),np.std(x2new),'mean and std',lc1_mean,lc1_rms)
 #import matplotlib.pylab as plt
 #plt.plot(x1)
 #plt.show()
 #input()
 
 x2op = np.interp(time1,time2,x2new)
 

 #try interpolating errors if present
 if (custom_mean != []):
  rms1 = custom_rms[0]
  rms2 = custom_rms[1]
 else:
  rms1 = lc1_rms
  rms2 = lc2_rms
   
 try:
  sig2op = np.interp(np.array(time1,dtype='float'),np.array(time2,dtype='float'),np.array(sig2,dtype='float'))*rms1/rms2
 except:
  print('no uncertainties given')
  sig2op = x2op*0 + 1.0
 
 
 
 
 
 
 
 #return the transformed and interpolated light curve in the same format it entered 
 #data frame or numpy array
 if (type(lc2) == pd.core.frame.DataFrame):
  lc2_op = pd.DataFrame(lc2)
  lc2_op.values[:,0] = time1in
  lc2_op.values[:,1] = x2op
  try:
   lc2_op.values[:,2] = sig2op
  except:
   print('no uncertainties given')
   
 else:
  lc2_op = np.array(lc2)
  lc2_op[:,0] = lc1[:,0]
  lc2_op[:,1] = x2op
  try:
   lc2_op[:,2] = sig2op
  except:
   print('no uncertainties given')
   sig2op = x2op*0 + 1.0
   
   
   
   
   
   
   
 #also return an array in a format
 #ready to insert into 'vortexa_combine_signal.py function'  
 n1 = np.shape(x1)[0]
 x_combine = np.zeros((n1,2))
 noise_combine = np.zeros((n1,2))
 x_combine[:,0] = x1
 noise_combine[:,0] = sig1
 x_combine[:,1] = x2op
 noise_combine[:,1] = sig2
 
  
 return(lc2_op,x_combine,noise_combine)
 
 
 
 
 
 
 
 
 
 
 


r_out=0.01
r_in=0.1
inc=0
D=1
M=1*10**8
M_dot=1
R_star=3
size=10000
from trprof_3 import *


def r_sch(M):
	rs=2*const.G*M/const.c**2
	return(rs)

### PLot the spectra at severla redshifts
#redshifts=[0.1,0.15,0.2,0.3,0.5,1,2]
#Mass=[1*10**8,2*10**8,3*10**8,4*10**8,5*10**8]

rad=[100/const.ps*r_sch(const.Msun*M),1000/const.ps*r_sch(const.Msun*M),10000/const.ps*r_sch(const.Msun*M),100000/const.ps*r_sch(const.Msun*M)]
spectra=np.zeros((size,len(rad)))
w_l=np.zeros((size,len(rad)))
count=0
l=[]
for D in (rad):
	l.append(D/r_sch(const.Msun*M)*const.ps)
	temp=flat_disk(D,r_in,inc,1,M,M_dot,R_star)
	w_l[:,count]=temp[0]#np.log10(temp[0])
	spectra[:,count]=temp[1]#np.log10(temp[1])
	plot(np.log10(w_l[:,count]),np.log10(spectra[:,count]),linewidth=2,marker='+')
	count=count+1

text=temp[2]  ## plotting information


xlabel('log(Wavelength / m)')
ylabel('log(flux Wm^-2)')
xticks(fontsize=15)
yticks(fontsize=15)
#title ('Z = '+str(text[0])+' .  r_in/r_out (pc)=' +str(text[1])+'/'+str(text[2])+'.  M='+str(text[4])+'.  M dot='+str(text[3])+'.  Inc='+str(text[5]))
title('z=1 r_in (pc)= 3rs. M= '+str(text[4])+'/Msun.   M dot='+str(text[3])+'.  Inc='+str(text[5]))	

legend(l)
show()




# python function takes an input array calculates the mean, lower and upper quartiles
# ip: parm[:nparm], conlev the confidence level required i.e 0.68 for 1 sigma etc
# op: the median, lower and upper confidence regions
import numpy as np

def mymedquart(parm,conlev=0.68):
 
 parmsort = np.sort(parm)
 nparm = parmsort.shape[0]
 
 
 idxparmmed = nparm/2

# calculate the median  
 if (np.mod(nparm,2) ==0):
  parmmed = (parmsort[idxparmmed-1] + parmsort[idxparmmed])/2 
 else:
  parmmed = parmsort[idxparmmed]
#



 a = conlev*nparm/2
 idxlo = int(np.max((idxparmmed - a,0)))
 idxhi = int(np.min((idxparmmed + a,nparm-1)))
 
 return(parmmed,parmsort[idxlo], parmsort[idxhi])


# test below on 5/2/2014. seems to work
#dat = np.random.randn(1000)
#
#op = mymedquart(dat,0.68)
#
#from pylab import *
#
#hist(dat)
#
#vlines(op[1],0,1000)
#vlines(op[0], 0, 1000)
#vlines(op[2], 0, 1000)
#
#show()






import numpy as np
import matplotlib.pylab as plt
#code to test weather I have the correct formula to predict the peak position of 
#a sinusoid with power-law-variable frequency and amplitude

n = np.arange(10)
f_slope = 1.0

f0 = 1.0
r0 = 1.0
A0 = 0.1
amp_slope = 1.0
wav0 = 1./f0

bg0 = 1.0
bg_slope = 2.0

rp = (n*wav0*r0**f_slope)**(1./(f_slope+1.))


rpmax = np.max(rp)
x = np.linspace(0,rpmax,1000)

amp = A0*(x/r0)**amp_slope
f = f0*(x/r0)**f_slope
bg = bg0*(x/r0)**bg_slope

y = amp * np.cos(2*np.pi * f  * x) + bg




fig = plt.figure()
ax1 = fig.add_subplot(111)
ax1.plot(x,y)


yp = np.interp(rp,x,y)


ax1.scatter(rp,yp,color='r')
#ax2 = fig.add_subplot(212)
#ax2.scatter(n,rp)


plt.show()
















import numpy as np
import scipy
import scipy.optimize


#ingest set of data find median and uncertainties
#input dat array
# op dmed median of dat, dlo,dup lower and upper sig confidence limits (i.e for 1 sigma, set sig = 0.68)
# disg = (dup-dlo)/2


def stat_uncert(dat,sig=0.68):
 nd = len(dat)#.shape()
 
 dat_sort=np.sort(dat)
 dmed = np.median(dat_sort)
 
 f = (1.-sig/2)
 nup = int(np.ceil((1.-f)*nd))
 nlo = int(np.ceil(f*nd))
 
 
 dlo = dat_sort[nlo]
 dup = dat_sort[nup]
 dsig = np.abs((dup-dlo)/2)
 
 return(dmed,dsig,dlo,dup)
 
 



#return uncertainty in theta give cos(theta) and sig_(costheta)



def sigtheta(costheta, sd_ct):

 ct_mean = np.mean(costheta)
 
 a = np.arccos(ct_mean)
 degmean = 180./np.pi*a
 sdmean =  1./np.sin(a) * sd_ct * 180/np.pi
 
 return(degmean,sdmean)
 
 





# fit polynomial to data by orthogonalizing parameters

#n order of poly
#x,y,sig array of data

def polyfit(x,y,sig,n):
 
 sum1oversig = np.sum(1./sig)
 
 xorth = np.sum(x**n / sig**n) / sum1oversig  
 
 xsuborth = x - xorth
 
 
 
 #fit a nth order poly to xsuborth data
 
 if (n == 1):
  def func(x,a,b): 
   return(a+b*x)
 elif (n==2):
  def func(x,a,b,c):
   return(a+b*x+c*x*x)
 elif (n==3):
  def func(x,a,b,c,d):
   return(a + b*x+c*x*x + d*x*x*x)
 elif (n==4):
  def func(x,a,b,c,d):
   return(a + b*x+c*x*x + d*x*x*x + e*x*x*x*x*x)
 else:
  raise ValueError('This function only works with polynomials up to n=4')
 
 
 
 fit_coef=scipy.optimize.curve_fit(func,np.array(x)-xorth,np.array(y),sigma=np.array(sig))
 sig_coef = np.array(())
 
 for i in range(n+1):
  sig_coef=np.append(sig_coef,np.sqrt([fit_coef[1][i,i]]))
  
 fit_coef = fit_coef[0]
 return(fit_coef,sig_coef,xorth)
  
 
 
 
 

#bayesian information criterion data x, model y,sig, k free params
def bic(x,y,sig,k):
 
 xsuby = x-y
 xsuby2 = xsuby*xsuby
 sig2 = sig*sig
 n=np.shape(x)[0]
 
 chisq = np.sum(xsuby2/sig2)
 
 bic = chisq + k*np.log(n) + n*np.log(2*np.pi) + np.sum(np.log(sig2))
 
 return(bic)
 

 
 
 
#convert uncertainty in  log or cos to real parameter and back again
#convert mean parameter (aveold) and uncertainty sdold and mean both from (1) and to (-1) log par and uncertainty, 
#and from (2) and to (-2) cosine par with uncertainty



def myconvlogsig(iversion,aveold,sdold):


 rln10 = np.log(10.)
 deg2rad = np.pi/180


#if going from log to real
 if (iversion == 1):
  a = 10**aveold
  avenew = a*(1. + 0.5*sdold*sdold*rln10*rln10)
  sdnew  = rln10*a * sdold



 #!! real to log
 elif (iversion == -1):

  a = alog10(aveold)
  avenew = a - 0.5*sdold*sdold/(rln10*aveold*aveold)
  sdnew  = 1./(rln10*aveold) * sdold







#!! cos to real
 elif (iversion ==2):
  print 'mystats doing fuck all'






# real to cos
 elif (iversion == -2):
  print 'mystats doing fuck all'
 
 

 else:
  print 'mystats.f90: iversion',iversion,' is not an option!!'
  exit
  
 
 a = cos(aveold*deg2rad)

 avenew = a*(1. - 0.5*sdold*sdold)
 sdnew  = sin(aveold*deg2rad)*sdold






 return(avenew,sdnew)
 
 
 
 
#function to return the fwhm of a distribution if funny shape just use ends 
def fwhm(t,x):
 xmin = np.min(x)
 xmax = np.max(x) - xmin
 xnew = (x - xmin)/xmax
 idx  = np.where(xnew > 0.5)[0]
 tlo  = t[idx[0]]
 thi  = t[idx[-1]]
 return(thi - tlo)
 
def fwhmall(t,x):
 xmin = np.min(x)
 xmax = np.max(x) - xmin
 xnew = (x - xmin)/xmax
 idx  = np.where(xnew > 0.5)[0]
 tlo  = t[idx[0]]
 thi  = t[idx[-1]]
 return(tlo,thi)
 
 
 
 
 


#manually written code to obtain the fourier transform of a real input signal
#it also does the inverse fft too. Will hopefully code this to give me the uncertainties
# of the spectrum given error bars on the signal. numpy and scipy's code does not do this.
#Thie FT gives both plus and - frequencies. The amplitude shown on the (e.g +ve frequency end) plots are half 
#the true amplitude as both +ve and -ve terms contribute half to the true amplitude

from pylab import *
import numpy as np

twopi=2*np.pi



# subroutine to calculate the fourier transform and uncertainties of an input data set
# this will go into the routine
# INPUT: x[:N],y[:N] signal
# OUTPUT: ftreal,ftimag,ftabs, sigftreal,sigftimag,sigftabs the sin and cosines amplitudes,and absolute amplitude spectrum and uncertaintes of the signal
# freq[:N] the frequencies of the signal


def myft(x,y,sigy=0):
    N=y.shape[0]
    dt = (x[-1] - x[0])/(N -1)
    if (isinstance(sigy,list)):
     sigyin = sigy
    else:
     sigyin = np.zeros(N)
     
    ftreal=np.zeros(N)
    ftimag=np.zeros(N)
    ftabs=np.zeros(N)
    sigftreal=np.zeros(N)
    sigftimag=np.zeros(N)
    sigftabs=np.zeros(N)

# calculate the real, imaginary, absolue, and uncertainties in the amplitude spectrum    
    for ik in range(N):
    
        w = twopi*(ik)*(np.arange(N)) /N
        cw = np.cos(w)
        sw = np.sin(w)
        
        realik = np.sum( y[:] * cw ) /N
        realik2 = realik*realik
        sigrealik = np.sqrt( np.sum( (cw*sigyin)**2) )/N
        sigrealik2 = sigrealik*sigrealik
        
        imagik = -np.sum( y[:] * sw ) /N
        sigimagik = np.sqrt( np.sum( (sw*sigyin)**2) )/N
        imagik2 = imagik*imagik
        sigimagik2 = sigimagik*sigimagik
        
        ftreal[ik] = realik
        sigftreal[ik]=sigrealik
        
        ftimag[ik] = imagik
        sigftimag[ik]= sigimagik
        
        ftabs[ik] = np.sqrt(realik2 + imagik2)
        sigftabs[ik] = 2*np.sqrt( realik2*sigrealik2 + imagik2*sigimagik2 )

    freq = np.fft.fftfreq(N,d=(N-1.)/N*dt)
    
    return(ftreal,sigftreal,ftimag,sigftimag,ftabs,sigftabs,freq)

#dftreal=1.*ftreal
#dftimag=1.*ftimag
#NT = N    
#t=1.*x
#tlo=t[0]
#thi=t[-1]  


#! subroutine to calculate the inverse fourier transform of an input set of real,
#! and imaginary fourier coefficients and a desired time grid to evaluate the ift at.
# inputs tlo,thi: first and last data point on the time series, dt: desired precision of output signal  
# outputs: tmod (the modified time array), oprealplot (the function after inverse ft)
  
def myift(ftreal,ftimag,tlo,thi,dtmod=-1):    
    tlen = thi - tlo 
    if (dtmod == -1):
        dtmod = tlen/(ftreal.shape[0]-1)/10
    
    
    
    NT = ftreal.shape[0]
    dt = tlen/(NT-1) # the separation between
    #dtmod = dt/10
    #NTmod = np.int(tlen/dtmod +1 )
    tmod = np.arange(tlo,thi+dtmod,dtmod)
    NTmod = np.int(tmod.shape[0])
#    print NTmod
    opreal=[]
    Ntfft = NTmod
    dtfft = (NT-1.)/(NTmod-1)
    tfft = np.arange(0,NT-1+dtfft,dtfft)
#    print tfft.shape[0]
#tfft = np.arange(NT)
#t_op  = np.arange(tlo,thi+dtmod,dtmod)

    #the NT/tfft[-1] is introduced to fix a small offset with the test lightcurve after the ift. NOT sure why this happened
    freq = np.fft.fftfreq(NT,d=NT/tfft[-1]) # give me the frequencies of the fourier transofmr
    wfft = 2*twopi*freq

#    oprealplot=[]
#    for it in range(NT):
#        w = twopi*np.arange(NT)*it /NT
#        sumreal = np.sum( ftreal[:NT]*np.cos(w) ) - np.sum( ftimag[:NT]*np.sin(w) )
#        opreal.append(sumreal)
    
    oprealplot=np.zeros(NTmod)
    #print NTmod,tmod.shape[0]
    for it in range(NTmod):
        oprealplot[it] = oprealplot[it]+np.sum( ftreal*np.cos(wfft*tfft[it]) - ftimag*np.sin(wfft*tfft[it]) )

    return(tmod,oprealplot)

    

#tlo=10.0
#thi=20.0
#dt=1.0
#x=np.arange(tlo,thi+dt,dt)
#y=1*np.cos(twopi*x) + 1*np.sin(twopi*2*x) + 4
#nx=x.size
#sigy=np.ones(nx)
#
#dat = myfft(x,y,sigy)
#realamp = dat[0]
#sigrealamp = dat[1]
#imagamp = dat[2]
#sigimagamp = dat[3]
#
#absamp = dat[4]
#sigabsamp=dat[5]
#freq=dat[6]
#
#errorbar(freq,absamp,sigabsamp)
#show()
#
##plot(y)
##plot(opreal)
##plot(tfft,oprealplot)
#
##show()
#    
#dat= myift(realamp,imagamp,tlo,thi)    
#plot(dat[0],dat[1])
#plot(x,dat[2])
#plot(x,y)
#show()
#

#    


import numpy as np
from pylab import *



G=6.673e-11
h=6.626e-34
k=1.38e-23
c=2.9979e8
pi=np.pi
msun=2e30
year=3600*24.*365
day=year/365
def taumeanface(lam,mmdot):
    
    x=lam*1e-10
    y=mmdot*msun*msun/year
    tdel=((45*G*y*x**4)/(16*pi**6 * c**5 *h))**(1./3)
    return(tdel/day)



def dlflat(mmdot,wav,cosi,fvb,fvf):

    epsilon=fvf/fvb
    b=(1.-epsilon)/(1.-epsilon**1.5)
    
    taucent=taumeanface(mmdot,wav)
    print mmdot,taucent,wav
    
    dlf = 6.3 * taucent * (wav/1.e4)**(-1.5) * 1/sqrt(fvb/cosi) * b*b

    return(dlf)




dat=np.loadtxt('outputpars.dat')
nwav=6
ncol=dat[:,0].shape[0]
wav=np.zeros(nwav)
fvb=np.zeros(nwav)
fvf=np.zeros(nwav)
mmdot=dat[:,2]

wav[:]=1367.20000, 3556.50928, 4702.48389, 6175.57031, 7489.96973, 8946.70312
fvb[:]=3.4,3.6,1.14,1.18,1.05,1.20
fvf[:]= 2.2,2.4,0.9,1.0,0.90,0.95

dl=np.zeros((ncol,nwav))
cosi=1.0
for i in range(ncol):
    for i2 in range(nwav):
         dl[i,i2]=dlflat(mmdot[i],wav[i2],cosi,fvb[i2],fvf[i2])
         
             
    



#### code to randomly resample a set of input light curves

##avesamp is the average length of time between the random samples

import numpy as np
import os
import sys

def myresample(dir,fname,dtave,sampmin=0.8,sampcode=2):
 dirpy=os.getcwd()
 #dir   = sys.argv[1] #the directory storing the light curves e.g '../fort/fortcode/fakelc/kep_18mar'
 #fname = sys.argv[2] # a list of the files .e.g ['file1.dat','file2.dat','...'] etc
 #dtave = sys.argv[3] #e.g 0.5 will resample with mean half day cadence
 #sampmin   = 0.8
 #sampcode = 2
 
 #!!### user arguments above. Don't change sampcode or sampmin unless you know what they do (I don't and I wrote the code).###
 
 os.chdir(dir)
 
 Nfile=len(fname)
 
 for ifile in range(Nfile):
     dat=np.loadtxt(fname[ifile])
     t=dat[:,0]
     x=dat[:,1]
     sig=dat[:,2]
     Ndat=t.shape[0]
     dt = (t[-1] - t[0])/(Ndat-1)
     
 # below are two versions of the code (the 2nd should be more sophisticated and consider the approximate spacing between each point when making its idxsamp selection    
     if sampcode == 1:
     	nidx=(1.-sampmin)*np.random.ranom_sample(1)[0]+sampminn
     	idxsamp=np.random.rand(low=0,high=Ndat,size=nidx)
     	datsamp=np.zeros((nidx,3))
     	datsamp[:,0]=t[idxsamp]
     	datsamp[:,1]=x[idxsamp]
     	datsamp[:,2]=sig[idxsamp]
     	
     elif sampcode == 2:
         idxcount=0
         tthen=t[0]
         idxsamp=[]
         while (idxcount < Ndat) & (tthen < t[-1]):
             
 
             a = np.random.randn(1)*dt
             
             tnow = tthen + dtave + a
             idxsamp.append(np.abs(t-tnow).argmin()) ## index of closest time to tnow
             
             tthen=tnow
             idxcount=idxcount+1
             
         idxsamp=np.array(idxsamp)
         datsamp=np.zeros((idxsamp.shape[0],3))
         datsamp[:,0]=t[idxsamp]
         datsamp[:,1]=x[idxsamp]
         datsamp[:,2]=sig[idxsamp]
 
 
 
     np.savetxt('resamp_'+fname[ifile],datsamp)   
 
 os.chdir(dirpy) # change back to python directory
 return()


# code to load and input light curve, and calculate the fourier transform and power spectrum, plotting the output on the plot.

import numpy as np
import os
import pylab as plt
from myoptscal import *
import scipy
import myfft

pwd = os.getcwd()
print 'Current directory :- ', pwd
dir = raw_input('Please input directory :- ')
os.chdir(dir)

nfiles = np.int(raw_input('Please input number of LC to analyse :- '))
fname=[]
for i in range(nfiles):
    print 'Please enter file',i+1,'of',nfiles,' :- '
    file = raw_input()
    fname.append(file)



#### hardwired parms

twopi=2*np.pi
nalong=1
include_phase = 0 # 0 or 1 depending if you want to include the phase spectrum in the plot
if (include_phase == 0):
    nvert = 2
else:
    nvert = 3

# set up the plot window
properdtmin = 0.2
fontsize=12

autoylim = 0
ylim=[-6,3]

col=['r','b','g','c','o','p','k','y']
fig=plt.figure()
fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.3)
font = {'family' : 'normal',
        'weight' : 'normal',
        'size'   : fontsize}

plt.rc('font', **font)
loge10 = np.log(10)

##### end of hardwired parms

for i in range(nfiles):
    dat =np.loadtxt(fname[i])
    t   =dat[:,0]
    x   =dat[:,1]
#    x=x-x.mean()
    sig =dat[:,2]
    

    nt  = t.shape[0]
    
    tfft=1.*np.arange(nt)
    ntmod = nt*10
    tlo=t[0]
    thi=t[-1]
    tlen = thi-tlo
    dt = tlen/(nt-1)
    dtmodplot = tlen/(ntmod-1)
    tmodplot = np.arange(t[0],t[-1],dtmodplot)  
    ntmod=tmodplot.shape[0]
    xmodplot = np.zeros(ntmod)
    
    #nt=tlen1/dt1 = tlen2/dt2

    ntfftmod = 1.*ntmod
    dtfftmod = (nt-1)/tlen * dtmodplot
    tfftmod=np.arange(tfft[0],tfft[-1]+dtfftmod, dtfftmod)

    
#    ntmodplot = (tmod[-1]-tmod[0])/dtmod + 1
    
    
#    tmodplot = 
#    ntmod=tmod.shape[0]

#calculate the fourier transform of the data must be evenly spaced, if not evenly spaced, interpolate.
    
    
    ft = np.fft.fft(x) / nt
    
    
    ftreal = np.real(ft)
    ftimag = np.imag(ft)
    freq = np.fft.fftfreq(nt,d=dt)
    w = twopi*freq
    ftabs = np.abs(ft)
    
    for it in range(ntmod):
        xmodplot[it] = xmodplot[it]  + np.sum( ftreal*np.cos(w*tfftmod[it]) - ftimag*np.sin(w*tfftmod[it]) )
    
#    xmodplot = xmodplot
 
 
 
    ft = myfft.myft(t,x,sig)
    ftabs = ft[4]
    sigftabs=ft[5]
    freq = ft[6]
    
    ift = myfft.myift(ftreal,ftimag,tlo,thi)   
    tmodplot = ift[0]
    xmodplot = ift[1]

#this is for the plotting
    ax1=fig.add_subplot(nvert,nalong,0)             # plot the light curve and model fit
    ax1.set_xlabel('time (days)')
    ax1.set_ylabel(r'f$_{\nu}$(t)')
    ax1.errorbar(t,x,sig,linestyle='None',color=col[i])
    ax1.plot(tmodplot,xmodplot,color=col[i])
   
    ax1=fig.add_subplot(nvert,nalong,1)             # plot the amplitude spectrum
    ax1.set_xlabel('frequency (cyc/day)')
    ax1.set_ylabel(r'$Amp^2$')
#    ax1.set_yscale('log', basey=10)
    ax1.set_xscale('log', basex=10)
    if (autoylim == 0):
        ax1.set_ylim(ylim[0],ylim[1])
    ax1.errorbar(freq,np.log10(ftabs),sigftabs,linestyle='None',marker='x',color=col[i])

ax1.legend(fname)

os.chdir(pwd) #change back to initial directory    

fig.show()
#fig.savefig('mydftfig.png')
       
    

#    ax1=fig.add_subplot(nalong,nvert,idxinc)
#    ax1.set_ylabel(r'$\psi$($\tau$ | $\lambda$)')
#    ax1.set_xlabel(r' $\tau$ (days)')
#    ax1.errorbar(t,x,sig,linestyle=None,color=col[i])



import os
import numpy as np
from pylab import *
# this code takes an input light curve
# truncates it into new light curves of length specified by the user.
# resaves the files

# hardwired options
equaldat = 1 #if equaldat = 1, the code will try to pick roughly even chunks but force them all to have the same number of data points

# filter twist th long kepler light curve periodically rotates, if filtertwist = 1, the light curve will be divided into segments which do not have a break in the filter 
filtertwist = 1

dir_default = os.getcwd()
dir = raw_input('Enter directory of data: ')
if not dir:
    dir = dir_default


os.chdir(dir)
fname = raw_input('Enter file name: ')




dat = np.loadtxt(fname)
x   = dat[:,0]
y   = dat[:,1]
sig = dat[:,2]

ndat = x.shape[0]

xdiff = np.abs(y[1:] - y[:-1])
xdiffmean = np.mean(xdiff)
xdiffsort = np.sort(xdiff)

nsections = 12
nchops = nsections - 1
cutoff = xdiffsort[-nchops:]
cutoff = np.insert(cutoff,0,0)

#idxchop = np.where( xdiff > cutoff)[0]#xdiffmean*10)[0]
#idxchop = np.insert(idxchop,0,0)

#nsections = idxchop.shape[0]
dirsave = 'chopped_mean'
direxist = os.path.isdir(dirsave)

if (direxist == True):
    os.system('rm -rf '+dirsave)

os.mkdir('chopped_mean')
os.chdir('chopped_mean')
for i in range(nsections-1):

    if (i == 0):
        idxlo = 0
    else:
        idxlo = np.where(xdiff == cutoff[i])[0]
    
    idxhi = np.where(xdiff == cutoff[i+1])[0]

    xnew = x[idxlo:idxhi]
    ynew = y[idxlo:idxhi]
    signew = sig[idxlo:idxhi]
    nnew = xnew.shape[0]
    
    datnew = np.zeros((nnew,3))
    datnew[:nnew,0] = xnew
    datnew[:nnew,1] = ynew
    datnew[:nnew,2] = signew
    np.savetxt('chopped_'+str(i)+'.dat',datnew)
    
    errorbar(xnew,ynew,signew,linestyle='None')
    savefig('chopped_'+str(i)+'.png')
    clf()
    
os.chdir('../')



import numpy as np
import matplotlib.pylab as plt
import vortexa_polyfit as vpf
import vortexa_rw_simple as vrw




def rwp(t,y,tgrid=[],order=3,custom_freqs = []):

 nt = np.shape(t)[0]
 if (tgrid == []):
  tg = np.array(t)
 else:
  tg = tgrid
  
 si = np.ones(nt)  
  
  
 #fot the polynomial
 poly = vpf.fit(t,y,si,order=order,xgrid=tg,confidence=0.3173,nits=20000,figure_title='')
 #return(yg_med,yg_lo,yg_hi,cov,cisq,cisq_red,bic,aic,rmd)
 ygp = poly[0]
 sgp = (poly[2] - poly[1])/2
 #interpolate the poly model onto data
 yp_itp = np.interp(t,tg,ygp)
 
 
 #subtract the poly model from the data
 ysub = y - yp_itp
 
 #fit the random walk to the polynomial-subtracted data
 rw = vrw.rw(t,ysub,tgi = tg,fbreak=-1,custom_freqs = custom_freqs)
 #return(tgrid,ygridop,f,ckout,skout)
 a = rw[1]
 ygrwlo,ygrwm,ygrwhi = a[:,0],a[:,1],a[:,2]
 sgrw = (ygrwhi - ygrwlo)/2
 
 
 
 #combine the two signals
 yg_tot = ygrwm + ygp
 sg_tot = np.sqrt(sgrw**2 + sgp**2)
 
 
 #fit statistic - autocorrelation function
 yg_tot_itp = np.interp(t,tg,yg_tot)
 rmd = np.corrcoef(yg_tot_itp,y)[0,1]
 
 
 
 #return the output
 return(yg_tot,sg_tot,rmd)












##test model on fake data
#
#from mylcgen import *
#from myrandom import *
#import myresample as mrs
#timeall = []
#sigall  = []
#yall    = []
#
#nlc = 1
#
#tlo = 0.0
#thi = 100
#dtave = 1.0
#dtres = 0.1
#
#for i in range(nlc):
# #generate test light curve and add noise
# datpre      = mylcgen(tlo=tlo,thi=thi,dt=dtres,iseed=132423)
# npre     = np.shape(datpre[:,0])[0]
# datmean  = np.std(datpre[:,1])
# snow =  np.ones(npre)/10*datmean 
# dat = mrs.myresample(dir='',fname=[''],dtave=dtave,sampmin=0.5,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],snow)).T)
# ndat = np.shape(dat[:,0])[0]
# sig = dat[:,2]
# for i in range(ndat):
#  dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
# sigall.append( sig )
# yall.append( dat[:,1] +50 )
# timeall.append( dat[:,0] )
# 
# 
#y = yall[0]
#t = timeall[0]
#s = sigall[0]
# 
# #dat =mylcgen(tlo=tlo,thi=thi,dt=dtave,iseed=132423)
# #nd = np.shape(dat[:,0])[0]
# #timeall = [dat[:,0]]
# #yall = [dat[:,1]]
# #sigall = [np.ones(nd)/10*datmean ]
# 
#y = y + 0.0*t**2
#print('going in')
#forecast = 50
#dtm = 0.4
#tmod = np.arange(tlo-forecast,thi+forecast,dtm)
#ntmod = np.shape(tmod)[0]
#xg = np.zeros((ntmod,3))
#ygrid,siggrid,rmid = rwp(t,y,tgrid=tmod,order=3)#rw(t,y,si=s,tgi = tmod,fbreak = 0.00005)
#xg[:,1] = ygrid
#xg[:,0] = ygrid - siggrid
#xg[:,2] = ygrid + siggrid
#
#
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.errorbar(t,y,s,ls='')
#ax1.fill_between(tmod,xg[:,0],xg[:,2],alpha=0.5)
#
#plt.show()
##from vortexa_mod_randomwalk import *
##a = fitrw([t],[y],[s],plot_tit='sh


import numpy as np

#inputs r(n,3) position vectors of all elements 
#a(n) area of elements
#norm(n, 3) normal vector for each surface element, 
#obs(3) unit vector to the oobserver

#returns
#shadow(n) integer vecor reads 1 if surface element can be seen or 0 if in shadow



#check all vectors are unit
def shadow(r,a,norm,obs):
 rmag    = np.sqrt(np.sum(r**2,axis=1))
 ru      = r/rmag[:,None]
 
 normmag = np.sqrt(np.sum(r**2,axis=1))
 normu   = norm/normmag[:,None]
 
 obsmag  = np.sqrt(np.sum(obs**2))
 obsu    = obs/obsmag
 
 nr = np.shape(r[:,0])[0]
 
 shadow = np.ones(nr)
 
 for i_i in range(nr):
  
  d_esubi = obs - r[i_i,:]
  mag_esubi = np.sqrt(np.sum(d_esubi**2))
  
  
  if (a[i_i] <= 0):#exclude lampposts
   shadow[i_i] = 0
   break
  for i_n in range(nr):
   if (i_i == i_n):#dont check point with itself
    continue
   if (a[i_n] <= 0):#exclude lampposts
    continue
   
   d_esubn = obs - r[i_n,:]
   mag_esubn = np.sqrt(np.sum(d_esubn**2))
   
   #if  closer to observer than n, dont bother with rest of loop because i cant be blocked by n 
   if (mag_esubn > mag_esubi):
    continue 
   
   d_nsubi = r[i_n,:] - r[i_i,:]
   mag_nsubi = np.sqrt(np.sum(d_nsubi**2))
   
   
   u_nsubi   = d_nsubi/mag_nsubi
   u_esubn   = d_esubn/mag_esubn
   
   #print 'predot left', u_nsubi*u_esubn
   dotleft = np.sum(u_nsubi*u_esubn)
   sinleft = np.sin(np.arccos(dotleft))
   
   left = mag_nsubi*sinleft#dotleft
   
   dotright = np.sum(u_esubn*normu[i_n])
   right = np.abs(dotright*np.sqrt(a[i_n]))
   
   
   if (left != left):
    print mag_nsubi,dotleft,u_nsubi,u_esubn,d_nsubi,i_i,i_n, 'problem with shadow code myshadow.py'
    print d_nsubi,mag_nsubi,r[i_n,:], r[i_i,:]
    raw_input()
   print i_n,r[i_n]
   print i_i,r[i_i]
   print i_i,left,right
   
   #right is the linear component of the area element projected into the direction of element n toward the observer, left is the distance between the line of site of n to point i, need right > left for point to be in shadow 
   if (right > left):
    shadow[i_i] = 0
    break
   
 return(shadow)
   
  
  
  
  
  
#code to test the shadow code
r = np.array(([0.,1.,0.],[0.,2.,0.]))
a = np.array([1.,1.])
norm = np.array(([0.,1.,0.],[0.,1.,0.]))
obs = np.array([0.,50.,0.])
idxshadow = shadow(r,a,norm,obs)







  


### subroutine to call a light curve and construct a power spectrum from the data
# NOTE when specifying the w(iw) = wlo + dw*(iw-1), the -1 is very important, if you dont include it, the model will not fit the data very well, (see you 1st year report for an example of what happens if you leave off the - 1.

# snippet from fortran code !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#NW=ceiling(0.5*NT)
#whi=pi/dt
#wlo=twopi*1/tlen!*0.01
#dw=(whi-wlo)/(NW-1)
#wlo=dw
#NW=ceiling((whi-wlo)/dw)-2


#allocate(w(NW),dtau(NW),sigdtau(NW))
#do i=1,NW
#w(i)=wlo+dw*(i-1)
#enddo
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

import numpy as np
import os
import pylab as plt
from myoptscal import *

pwd = os.getcwd()
print 'Current directory :- ', pwd
dir = raw_input('Please input directory :- ')
os.chdir(dir)

nfiles = np.int(raw_input('Please input number of LC to analyse :- '))
fname=[]
for i in range(nfiles):
    print 'Please enter file',i+1,'of',nfiles,' :- '
    file = raw_input()
    fname.append(file)



#### hardwired parms


nalong=1
include_phase = 0 # 0 or 1 depending if you want to include the phase spectrum in the plot
if (include_phase == 0):
    nvert = 2
else:
    nvert = 3

# set up the plot window
properdtmin = 0.2
fontsize=12


autoylim = 0
ylim=[-6,3]

col=['r','b','g','c','o','p','k','y']
fig=plt.figure()
fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.3)
font = {'family' : 'normal',
        'weight' : 'normal',
        'size'   : fontsize}

plt.rc('font', **font)
loge10 = np.log(10)

##### end of hardwired parms

for i in range(nfiles):
    op = mypowerspecdef.mypowerspec(fname[i])
#    dat=hesfit(x,y,sig,p):
#    dat[0][0]

#this is for the plotting
    ax1=fig.add_subplot(nvert,nalong,0)             # plot the light curve and model fit
    ax1.set_xlabel('time (days)')
    ax1.set_ylabel(r'f$_{\nu}$(t)')
    ax1.errorbar(t,x,sig,linestyle='None',color=col[i])
    ax1.plot(tmod,xop,color=col[i])
    
    ax1=fig.add_subplot(nvert,nalong,1)             # plot the amplitude spectrum
    ax1.set_xlabel('frequency (cyc/day)')
    ax1.set_ylabel(r'$Amp^2$')
#    ax1.set_yscale('log', basey=10)
    ax1.set_xscale('log', basex=10)
    ax1.errorbar(f[:nf],ampspeclog,sigampspeclog,linestyle='None',color=col[i])
    if (autoylim == 0):
        ax1.set_ylim(ylim[0],ylim[1])
    
    
    if (include_phase == 1):
        ax1=fig.add_subplot(nvert,nalong,2)             # plot the phase spectrum
        ax1.set_xlabel('frequency (cyc/day)')
        ax1.set_ylabel('phase')
        ax1.errorbar(f[:nf],phasespec,sigphasespec,linestyle='None',color=col[i])

ax1.legend(fname)

	# save the output
opdat = np.zeros((nf,3))
opdat[:,0]=np.log10(f[:nf])
opdat[:,1]=ampspeclog
opdat[:,2]=sigampspeclog
np.savetxt('mypowerspecop_'+fname[i],opdat)

os.chdir(pwd) #change back to initial directory    




fig.show()

#fig.savefig('mypowerspecfig.png')
       
    

#    ax1=fig.add_subplot(nalong,nvert,idxinc)
#    ax1.set_ylabel(r'$\psi$($\tau$ | $\lambda$)')
#    ax1.set_xlabel(r' $\tau$ (days)')
#    ax1.errorbar(t,x,sig,linestyle=None,color=col[i])





"""
    Module for computing The Hartigans' dip statistic
    The dip statistic measures unimodality of a sample from a random process.
    See: 
    Hartigan, J. A.; Hartigan, P. M. The Dip Test of Unimodality. The Annals 
    of Statistics 13 (1985), no. 1, 70--84. doi:10.1214/aos/1176346577. 
    http://projecteuclid.org/euclid.aos/1176346577.
"""

import numpy as np
import collections

def _gcm_(cdf, idxs):
    work_cdf = cdf
    work_idxs = idxs
    gcm = [work_cdf[0]]
    touchpoints = [0]
    while len(work_cdf) > 1:
        distances = work_idxs[1:] - work_idxs[0]
        slopes = (work_cdf[1:] - work_cdf[0]) / distances
        minslope = slopes.min()
        minslope_idx = np.where(slopes == minslope)[0][0] + 1
        gcm.extend(work_cdf[0] + distances[:minslope_idx] * minslope)
        touchpoints.append(touchpoints[-1] + minslope_idx)
        work_cdf = work_cdf[minslope_idx:]
        work_idxs = work_idxs[minslope_idx:]
    return np.array(np.array(gcm)),np.array(touchpoints)

def _lcm_(cdf, idxs):
    g,t = _gcm_(1-cdf[::-1], idxs.max() - idxs[::-1])
    return 1-g[::-1], len(cdf) - 1 - t[::-1]

def _touch_diffs_(part1, part2, touchpoints):
    diff = np.abs((part2[touchpoints] - part1[touchpoints]))
    return diff.max(), diff

def dip(histogram=None, idxs=None):
    """
        Compute the Hartigans' dip statistic either for a histogram of
        samples (with equidistant bins) or for a set of samples.
    """
    if idxs is None:
        idxs = np.arange(len(histogram))
    elif histogram is None:
        h = collections.Counter(idxs)
        idxs = np.msort(h.keys())
        histogram = np.array([h[i] for i in idxs])
    else:
        if len(histogram) != len(idxs):
            raise ValueError("Need exactly as many indices as histogram bins.")
        if len(idxs) != len(set(idxs)):
            raise ValueError("idxs must be unique if histogram is given.")
        if not np.array_equal(np.msort(idxs), idxs):
            idxs_s = np.argsort(idxs)
            idx = np.asarray(idxs)[idxs_s]
            histogram = np.asarray(histogram)[idxs_s]

    cdf = np.cumsum(histogram, dtype=float)
    cdf /= cdf[-1]

    work_idxs = idxs
    work_histogram = np.asarray(histogram, dtype=float) / np.sum(histogram)
    work_cdf = cdf

    D = 0
    left = [0]
    right = [1]

    while True:
        left_part, left_touchpoints   = _gcm_(work_cdf - work_histogram, work_idxs)
        right_part, right_touchpoints = _lcm_(work_cdf, work_idxs)

        d_left, left_diffs   = _touch_diffs_(left_part, right_part, left_touchpoints)
        d_right, right_diffs = _touch_diffs_(left_part, right_part, right_touchpoints)

        if d_right > d_left:
            xr = right_touchpoints[d_right == right_diffs][-1]
            xl = left_touchpoints[left_touchpoints <= xr][-1]
            d  = d_right
        else:
            xl = left_touchpoints[d_left == left_diffs][0]
            xr = right_touchpoints[right_touchpoints >= xl][0]
            d  = d_left

        left_diff  = np.abs(left_part[:xl+1] - work_cdf[:xl+1]).max()
        right_diff = np.abs(right_part[xr:]  - work_cdf[xr:] + work_histogram[xr:]).max()

        if d <= D or xr == 0 or xl == len(work_cdf):
            the_dip = max(np.abs(cdf[:len(left)] - left).max(), np.abs(cdf[-len(right)-1:-1] - right).max())
            return the_dip/2, (cdf, idxs, left, left_part, right, right_part)
        else:
            D = max(D, left_diff, right_diff)

        work_cdf = work_cdf[xl:xr+1]
        work_idxs = work_idxs[xl:xr+1]
        work_histogram = work_histogram[xl:xr+1]

        left[len(left):] = left_part[1:xl+1]
        right[:0] = right_part[xr:-1]

def plot_dip(histogram=None, idxs=None):
    from matplotlib import pyplot as plt

    d,(cdf,idxs,left,left_part,right,right_part) = dip(histogram,idxs)


    plt.plot(idxs[:len(left)], left, color='red')
    plt.plot(idxs[len(left)-1:len(left)+len(left_part) - 1], left_part, color='gray')
    plt.plot(idxs[-len(right):], right, color='blue')
    plt.plot(idxs[len(cdf) - len(right) + 1 - len(right_part):len(cdf) - len(right) + 1], right_part, color='gray')

    the_dip = max(np.abs(cdf[:len(left)] - left).max(), np.abs(cdf[-len(right)-1:-1] - right).max())
    l_dip_idxs = np.abs(cdf[:len(left)] - left) == the_dip
    r_dip_idxs = np.abs(cdf[-len(right)-1:-1] - right) == the_dip
    print(the_dip/2, d)

    plt.vlines(x=idxs[:len(left)][l_dip_idxs], ymin=cdf[:len(left)][l_dip_idxs], ymax = cdf[:len(left)][l_dip_idxs] - the_dip)
    plt.vlines(x=idxs[-len(right):][r_dip_idxs], ymin=cdf[-len(right)-1:-1][r_dip_idxs], ymax = cdf[-len(right)-1:][r_dip_idxs] + the_dip)

    plt.plot(np.repeat(idxs,2)[1:], np.repeat(cdf,2)[:-1], color='black')
    plt.scatter(idxs, cdf)

    plt.show()


def crit_points(random_function, quantiles, sample_size, n_samples):
    """
        Compute the quantiles for the dip statistic for n_samples 
        samples of size sample_size from the random process given by 
        random_function.
        Parameters:
        random_function : a paramter-free function which returns random values.
        quantiles : a sequence of values between 0 and 1
        sample_size : the size of the samples to draw from random_function
        n_samples : the number of samples for which to compute dips
        Returns: a list such that the i'th value is the greatest dip observed
        such that the fraction of dips less than or equal to that value is less
        than the i'th value from quantiles.
    """
    data = [[random_function() for _ in range(sample_size)] for __ in range(n_samples)]
    dips = np.array([dip(idxs=samples)[0] for samples in data])
    
    return np.percentile(dips, [p * 100 for p in quantiles])


import numpy as np


#input light curves and caculate cross spec
#inputs t1,x1,t2,x2 arrays for the time and flux of input light curves
#output frequency array, phase array and phase lag (phase/(2pi*freq))

#NOTES if power_weighted == 1then create a transfer fucntion strength (psitaulam vs lag) where each 
#phase lag is weighted by the power A*B for each fourier frequency. SUM_1^NW A(iw)*B(iw) * phaselag
def mcs(t1,x1,t2,x2,power_weighted=1):

 tlo = min(np.min(t1), np.min(t2))
 thi = max(np.max(t1), np.max(t2))
 dt1 = np.mean(t1[1:] - t1[:-1])
 dt2 = np.mean(t2[1:] - t2[:-1])
 dt  = (dt1 + dt2)/2
 
 #perform interpolation
 titp = np.arange(tlo,thi+dt,dt)
 nt   = np.shape(titp)[0]
 x1itp = np.interp(titp,t1,x1)
 x2itp = np.interp(titp,t2,x2)
 
 #calculate cross spectrum
 ftx1 = np.fft.fft(x1itp)
 ftx2 = np.fft.fft(x2itp)
 conjx2 = np.conj(ftx2)
 cs = ftx1 * conjx2
 
 freq = np.fft.fftfreq(nt,d=dt)
 #calculate amplitudes of each fourier component
 amp1_ft = np.abs(ftx1)
 amp2_ft = np.abs(ftx2)
 
 
 #calculate the phase
 phi = np.arccos(np.real(cs/amp1_ft/amp2_ft))
 
 #calculate the phase lag
 taunu = phi/2/np.pi/freq
 
 psitaulam = []
 if (power_weighted ==1):
  psitaulam = amp1_ft*amp2_ft
  
 return(freq,taunu,phi,psitaulam)
 
 


import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import numpy as np

def mce(cov, pos, nstd=2, ax=None, **kwargs):
    """
    Plots an `nstd` sigma error ellipse based on the specified covariance
    matrix (`cov`). Additional keyword arguments are passed on to the 
    ellipse patch artist.

    Parameters
    ----------
        cov : The 2x2 covariance matrix to base the ellipse on
        pos : The location of the center of the ellipse. Expects a 2-element
            sequence of [x0, y0].
        nstd : The radius of the ellipse in numbers of standard deviations.
            Defaults to 2 standard deviations.
        ax : The axis that the ellipse will be plotted on. Defaults to the 
            current axis.
        Additional keyword arguments are pass on to the ellipse patch.

    Returns
    -------
        A matplotlib ellipse artist
    """
    
    def eigsorted(cov):
        vals, vecs = np.linalg.eigh(cov)
        order = vals.argsort()[::-1]
        return vals[order], vecs[:,order]

    if ax is None:
        ax = plt.gca()

    vals, vecs = eigsorted(cov)
    theta = np.degrees(np.arctan2(*vecs[:,0][::-1]))

    # Width and height are "full" widths, not radius
    width, height = 2 * nstd * np.sqrt(vals)
    ellip = Ellipse(xy=pos, width=width, height=height, angle=theta, **kwargs)

    ax.add_artist(ellip)
    return ellip
    
    
    
    
def mc_b(cov, pos, nstd=2, ax=None, **kwargs):

 return mce(cov, pos, nstd=nstd, ax=ax)


import numpy as np
import matplotlib.pyplot as pl
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.patches import Ellipse
import matplotlib.cm as cm
import scipy
import scipy.stats
try:
 import myconfidence_ellipse as mc_b
except:
 pass

def corner_1(xs, weights=None, labels=None, extents=None, truths=None,
           truth_color="#4682b4", scale_hist=False, quantiles=[],
           verbose=True, plot_contours=False, plot_datapoints=False,
           fig_ax=None,header=[],sigconts=[100.-68,100.-95,100-99.7],figname='histplot.pdf',skip=1,xti=[-1],yti=[-1], annotate=[],
           medplot=1,sigplot=0.68,sigmedann=1,sigmedann_pre_in=[],title='',sigmedann_post_in=[],xtxtcoord=0.95,ytxtcoord=0.95,
           specax=[],specaxlab=[],plot_corr=0,
           lbm_in = 0.65,ltr_in = 0.4,xlab_coord=-0.3,ylab_coord=-0.3,**kwargs):
    
    npar = np.shape(xs)[1]
    if (len(specax) == 0):
     specaxin = [[] for i in range(npar)]
    else:
     specaxin = list(specax)
    if (len(specaxlab) == 0):
     specaxlabin = [[] for i in range(npar)]
    else:
     specaxlabin = list(specaxlab)
    
    
    if (len(header) ==0):
     header=['']*npar
    
    """
    Make a *sick* corner plot showing the projections of a data set in a
    multi-dimensional space. kwargs are passed to hist2d() or used for
    `matplotlib` styling.

    Parameters
    ----------
    xs : array_like (nsamples, ndim)
        The samples. This should be a 1- or 2-dimensional array. For a 1-D
        array this results in a simple histogram. For a 2-D array, the zeroth
        axis is the list of samples and the next axis are the dimensions of
        the space.

    weights : array_like (nsamples,)
        The weight of each sample. If `None` (default), samples are given
        equal weight.

    labels : iterable (ndim,) (optional)
        A list of names for the dimensions.

    extents : iterable (ndim,) (optional)
        A list where each element is either a length 2 tuple containing
        lower and upper bounds (extents) or a float in range (0., 1.)
        giving the fraction of samples to include in bounds, e.g.,
        [(0.,10.), (1.,5), 0.999, etc.].
        If a fraction, the bounds are chosen to be equal-tailed.
        OR (1,1) if same number is chosen then min and max values are used as limits

    truths : iterable (ndim,) (optional)
        A list of reference values to indicate on the plots.

    truth_color : str (optional)
        A ``matplotlib`` style color for the ``truths`` makers.

    scale_hist : bool (optional)
        Should the 1-D histograms be scaled in such a way that the zero line
        is visible?

    quantiles : iterable (optional)
        A list of fractional quantiles to show on the 1-D histograms as
        vertical dashed lines.

    verbose : bool (optional)
        If true, print the values of the computed quantiles.

    plot_contours : bool (optional)
        Draw contours for dense regions of the plot.

    plot_datapoints : bool (optional)
        Draw the individual data points.
    
    annotate : list[...ni] of strings to add to each plot (each histogram plot) NOTE annotate and sigmedann put things in the same place
    
    
    sigconts : plot contour levels by default at 1, 2, 3 sigma... e.g [100.-68,100.-95,100-99.7]. set sigconts[0] = 1 to fit n ellipse to you probability distribution and fit contours that way (must set sigconts[0] = 1 to do this where 1 is the 1 sigma elipse fitted).
    
    fig_ax : matplotlib.Figure (optional) and axes [fig,axes] format
        Overplot onto the provided figure object.
    
    plot_corr : 1 to annotate plots with correlation coefficient
    
    medplot : 1 to annotate histograms with median line
    
    sigplot : >0 to annotate histograms with sigplot confidence intervals (e.g 0.68) for 1 sig confidence upper lower limits

    sigmedann : 1 to label the histogram plots with uncertainties and median 
    
    sigmedann_pre_in [ndim] n-dimensional list of strings to put infront of the uncertainty and median parameter estmates if annotating
    
    sigmedann_post_in [ndim] n-dimensional list of strings to put at end of the uncertainty and median parameter estmates if annotating
    
    lbm_in: size of left and bottom margains
    
    ltr_in: size of top and right margains
    
    xlab_coord: distance of x-axis label from outside edge of axis -ve means plot axis label outside of frame
    
    ylab_coord: distance of y-axis label from outside edge of axis -ve means plot axis label outside of frame
    """
    nconts = len(sigconts)
    
    #deal with 1d parameter arrays
    ndtemp = np.shape(xs)[1]
    if (ndtemp == 1):
     xs = xs[:,0]
    
    
    #deal with annotations of uncertainties
    if (sigmedann_pre_in == []):
     sigmedann_pre = ['']*ndtemp
    else:
     sigmedann_pre = sigmedann_pre_in
    if (sigmedann_post_in == []):
     sigmedann_post = ['']*ndtemp
    else:
     sigmedann_post = sigmedann_post_in
 



    
    # Deal with 1D sample lists.
    xs = np.atleast_1d(xs)
    if len(xs.shape) == 1:
        xs = np.atleast_2d(xs)
        
    else:
        assert len(xs.shape) == 2, "The input sample array must be 1- or 2-D."
        xs = xs.T
    #assert xs.shape[0] <= xs.shape[1], "I don't believe that you want more " \
    #                                   "dimensions than samples!"

    if weights is not None:
        weights = np.asarray(weights)
        if weights.ndim != 1:
            raise ValueError('weights must be 1-D')
        if xs.shape[1] != weights.shape[0]:
            raise ValueError('lengths of weights must match number of samples')

    # backwards-compatibility
    plot_contours = kwargs.get("smooth", plot_contours)
    
    
    
    K = len(xs)
    
    #print(K)
    factor = 1.0*6/K           # size of one side of one panel
    lbdim = lbm_in * factor   # size of left/bottom margin lbm_in = 0.65
    trdim = ltr_in * factor  # size of top/right margin #ltr_in = 0.4
    whspace = 0.05         # w/hspace size
    plotdim = factor * K + factor * (K - 1.) * whspace
    dim = lbdim + plotdim + trdim

    if fig_ax is None:
        fig, axes = pl.subplots(K, K, figsize=(dim, dim))
    else:
        fig = fig_ax[0]
        axes = fig_ax[1]
    #    try:
    #        axes = np.array(fig.axes).reshape((K, K))
    #    except:
    #        raise ValueError("Provided figure has {0} axes, but data has "
    #                         "dimensions K={1}".format(len(fig.axes), K))
    lb = lbdim / dim
    tr = (lbdim + plotdim) / dim
    fig.subplots_adjust(left=lb, bottom=lb, right=tr, top=tr,
                        wspace=whspace, hspace=whspace)

    if extents is None:
        extents = [[x.min(), x.max()] for x in xs]

        # Check for parameters that never change.
        m = np.array([e[0] == e[1] for e in extents], dtype=bool)
        if np.any(m):
            raise ValueError(("It looks like the parameter(s) in column(s) "
                              "{0} have no dynamic range. Please provide an "
                              "`extent` argument.")
                             .format(", ".join(map("{0}".format,
                                                   np.arange(len(m))[m]))))
    else:
        # If any of the extents are percentiles, convert them to ranges.
        for i in range(len(extents)):
            try:
                emin, emax = extents[i]
                #if only want to manually set limits for one dimension
                if (emin == emax):
                 extens[i] =(x[i].min(),x[i].max())
            except TypeError:
                q = [0.5 - 0.5*extents[i], 0.5 + 0.5*extents[i]]
                extents[i] = quantile(xs[i], q, weights=weights)
    
    
    
      
    
    
    
    
    
    for i, x in enumerate(xs):
        if (ndtemp == 1):
            fig = pl.figure()
            ax  = fig.add_subplot(111)
        else:
            ax = axes[i, i]
        # Plot the histograms.
        n, b, p = ax.hist(x, weights=weights, bins=kwargs.get("bins", 50),
                          range=extents[i], histtype="step",normed=1,
                          color=kwargs.get("color", "k"))
        
        
        
        if (len(annotate) > 0):
         ax.text(xtxtcoord,ytxtcoord,annotate[i],ha='right',transform=ax.transAxes)
        
        ax.set_title(header[i],loc='left')
        xlimlo = extents[i][0]
        xlimhi = extents[i][1]
        binwidth_x = (xlimhi-xlimlo)/kwargs.get("bins", 50)
        ylhist = list(ax.get_ylim())
        ylhist[1] = ylhist[1]*1.1
        ax.set_ylim(ylhist)
        
        
        
        
        
        
        if (xti == []):
         ax.set_xticklabels([])
        if (yti == []):
         ax.set_yticklabels([])
        
        if truths is not None:
            if (truths[i] != ''):
             ax.axvline(truths[i], color=truth_color)
 
        # Plot quantiles if wanted.
        if len(quantiles) > 0:
            qvalues = quantile(x, quantiles, weights=weights)
            for q in qvalues:
                ax.axvline(q, ls="dashed", color=kwargs.get("color", "k"))
            if verbose:
                print("Quantiles:")
                print(zip(quantiles, qvalues))

        # Set up the axes.
        ax.set_xlim(extents[i])
        if scale_hist:
            maxn = np.max(n)
            ax.set_ylim(-0.1 * maxn, 1.1 * maxn)
        else:
            ax.set_ylim(0, 1.1 * np.max(n))
        ax.set_yticklabels([])
        ax.xaxis.set_major_locator(MaxNLocator(5))

        
        
        #draw sigplot and medplot lines if optioned toggled on
        if ((medplot == 1 )or (sigplot > 0) or (sigmedann == 1)):
         ylnow   = list(ax.get_ylim())
         idxsort = np.argsort(x)
         nidx    = np.shape(idxsort)[0]
         xsort   = x[idxsort]
         
        if (medplot == 1 or sigmedann == 1):
         idmed = nidx/2
         xmed = xsort[idmed]
         if (medplot == 1):
          ax.plot([xmed,xmed],ylnow,color='k')
        if ((sigplot > 0) or (sigmedann == 1)):
         spp = np.abs(sigplot)
         if (np.abs(sigplot) >1):
          print('can only plot confidence region between 0 and 1. change sigplot lt 1')
          print('make it -1 gt sigplot lt 0 if dont want to plot uncertainties but still want to annotate them')
          spp = 0.68
         
         frac  = (1.-spp)/2
         idxlo = int(frac*nidx)
         idxhi = int((1.-frac)*nidx)
         xlo   = xsort[idxlo]
         siglo = xmed - xlo
         xhi   = xsort[idxhi]
         sighi = xhi  - xmed
         if (sigplot > 0):
          ax.plot([xlo,xlo],ylnow,ls='--',color='k')
          ax.plot([xhi,xhi],ylnow,ls='--',color='k')
        
        
        if ( sigmedann == 1):
         txt = r' '+sigmedann_pre[i]+' $='+str(np.round(xmed,2))+'^{+'+str(np.round(sighi,2))+'}_{-'+str(np.round(siglo,2))+'}$'+sigmedann_post[i]
         ax.text(xtxtcoord,ytxtcoord,txt,ha='right',transform=ax.transAxes)
        
        
         
        # Not so DRY.
        if i < K - 1:
            ax.set_xticklabels([])
        else:
            
            #change x axis labeling if custom option entered
            if (specaxlabin[i] != []):
             ax.set_xticklabels(specaxlabin[i])
             print('xtick labels set',specaxlabin[i])
             print(ax.get_xlim())
            [l.set_rotation(45) for l in ax.get_xticklabels()]
            if labels is not None:
                ax.set_xlabel(labels[i])
                ax.xaxis.set_label_coords(0.5, xlab_coord)

        
        
        #print 'FUCK OFFFFF dont get it',i,specaxin[i],ndtemp
        if (specaxin[i] != []):
             ax.set_xticks(specaxin[i])
             #print 'xticks set', specaxin[i]
        #need go no further if only varying one parameter
        if (ndtemp == 1):
         [l.set_rotation(0) for l in ax.get_xticklabels()]
         print(i,labels,'herere')
         if labels is not None:
          ax.text(0.5,-0.07,labels[i],va='top',ha='center',transform=ax.transAxes)
         if (figname != ''):
          #pl.tight_layout()
          pl.savefig(figname)
         return fig, ax
        
        for j, y in enumerate(xs):
            ax = axes[i, j]
            if j > i:
                ax.set_visible(False)
                ax.set_frame_on(False)
                continue
            elif j == i:
                continue

            hist2d(y[::skip], x[::skip], ax=ax, extent=[extents[j], extents[i]],
                   plot_contours=plot_contours,
                   plot_datapoints=plot_datapoints,
                   weights=weights, **kwargs)
            ylimlo = extents[j][0]
            ylimhi = extents[j][1]
            binwidth_y = (ylimhi-ylimlo)/kwargs.get("bins", 50)

 
            if (sigconts[0] == 1):
             cov = np.cov(y,x)
             mean = np.mean(y),np.mean(x)
             for scnow in sigconts:
              mc_b.mce(cov, mean, nstd=scnow,ax=ax,fill=False,edgecolor=kwargs.get("color", "k"),linewidth=2.0,color=kwargs.get("color", "k"))
            elif (sigconts[0] == 0):
             print('')  
            else:
             xtemp = y
             ytemp = x
             xlotemp = ylimlo
             xhitemp = ylimhi
             ylotemp = xlimlo
             yhitemp = xlimhi
             bwx     = binwidth_y
             bwy     = binwidth_x
             
             both = np.array((xtemp,ytemp)).T
             #both = np.array((x,y))
             #print('making pdf')
             pdf1=scipy.stats.kde.gaussian_kde(both.T)
             #print(ylimlo,ylimhi,binwidth_y,xlimlo,xlimhi,binwidth_x,extents)
             #q,w=np.meshgrid(np.arange(ylimlo,ylimhi,binwidth_y), np.arange(xlimlo,xlimhi,binwidth_x))
             q,w=np.meshgrid(np.arange(xlotemp,xhitemp,bwx), np.arange(ylotemp,yhitemp,bwy))
             r1=pdf1([q.flatten(),w.flatten()])
             #r1 = r1/np.max(r1)
             s1=[]
             #print('doing contours',nconts,sigconts,xlimlo,xlimhi,binwidth_x,ylimlo,ylimhi,binwidth_y,np.min(r1),np.max(r1))
             #print(pdf1)
             #print('pdf1')
             #raw_input()
             #print(q)
             #print('q')
             #raw_input()
             #print(w)
             #print('w')
             #raw_input()
             #print(r1)
             #print('r1')
             #raw_input()
             for ic in range(nconts):
              s1.append(scipy.stats.scoreatpercentile(pdf1(pdf1.resample(1000)), sigconts[ic]))
             r1.shape=(q.shape[0],q.shape[1])
             for ic in range(nconts):
              #ax.contour(np.arange(xlimlo,xlimhi,binwidth_x), np.arange(ylimlo,ylimhi,binwidth_y), r1, [s1[ic]])
              #print('contourminmax',xlimlo,xlimhi,ylimlo,ylimhi,np.min(r1),np.max(r1))
              #ax.contour(np.arange(xlimlo,xlimhi,binwidth_x), np.arange(ylimlo,ylimhi,binwidth_y), r1, [np.max(r1)/2],color='r',lwd=20)
              ax.contour(np.arange(xlotemp,xhitemp,bwx), np.arange(ylotemp,yhitemp,bwy), r1,[s1[ic]])
              #ax.text(0.5,0.5,'erewrw')
              #pl.show()
             
             
             
             
            if (xti == []):
             ax.set_xticklabels([])
            if (yti == []):
             ax.set_yticklabels([])
            
            if (plot_corr > 0):
             r_ann = np.corrcoef(np.array((y[::skip],x[::skip])))[0,1]
             nt = np.shape(y)[0]
             #for i in range(nt):
             # print(y[i],x[i])
             #print(r_ann)
             #print(np.corrcoef(np.array((y[1:],x[1:]))))
             #raw_input()
             ax.text(xtxtcoord,ytxtcoord-0.05,'R='+str(np.round(r_ann,2)),ha='right',transform=ax.transAxes)
             #ax.text(0.95,0.9,'R='+str(np.round(r_ann,2)),ha='right',transform=ax.transAxes)
            
            #if (len(annotate) > 0):
            # ax.text(0.95,0.9,annotate[i],ha='right',transform=ax.transAxes)
            
            if truths is not None:
                
                if (truths[i] != '' and truths[j] != '' ):
                 ax.plot(truths[j], truths[i], "s", color=truth_color)
                
                if (truths[j] != ''):
                 ax.axvline(truths[j], color=truth_color)
                
                if (truths[i] != ''):
                 ax.axhline(truths[i], color=truth_color)

            ax.xaxis.set_major_locator(MaxNLocator(5))
            ax.yaxis.set_major_locator(MaxNLocator(5))
           
           
           
           
           
           
           
           
           
           
           
           
           
           
            #print 'FUCK OFFFFF dont get it',i,specaxin[i],ndtemp
            if (specaxin[j] != []):
             ax.set_xticks(specaxin[j])
            if (specaxin[i] != []):
             ax.set_yticks(specaxin[i])

             #print 'xticks set', specaxin[i]



            if i < K - 1:
                ax.set_xticklabels([])
            else:
                if (specaxlabin[j] != []):
                 ax.set_xticklabels(specaxlabin[j])
                 #print 'xtick labels set',specaxlabin[i]
                 #print ax.get_xlim()

                [l.set_rotation(45) for l in ax.get_xticklabels()]
                if labels is not None:
                    ax.set_xlabel(labels[j])
                    ax.xaxis.set_label_coords(0.5, xlab_coord)

            if j > 0:
                ax.set_yticklabels([])
            else:
                if (specaxlabin[i] != []):
                 ax.set_yticklabels(specaxlabin[i])
                [l.set_rotation(45) for l in ax.get_yticklabels()]
                if labels is not None:
                    ax.set_ylabel(labels[i])
                    ax.yaxis.set_label_coords(ylab_coord, 0.5)

    
    if (title != ''):
     ax.set_title(title,loc='left')
     
    if (figname != ''):
     pl.savefig(figname)
    return fig,axes


def quantile(x, q, weights=None):
    """
    Like numpy.percentile, but:

    * Values of q are quantiles [0., 1.] rather than percentiles [0., 100.]
    * scalar q not supported (q must be iterable)
    * optional weights on x

    """
    if weights is None:
        return np.percentile(x, [100. * qi for qi in q])
    else:
        idx = np.argsort(x)
        xsorted = x[idx]
        cdf = np.add.accumulate(weights[idx])
        cdf /= cdf[-1]
        return np.interp(q, cdf, xsorted).tolist()


def error_ellipse(mu, cov, ax=None, factor=1.0, **kwargs):
    """
    Plot the error ellipse at a point given its covariance matrix.

    """
    # some sane defaults
    facecolor = kwargs.pop('facecolor', 'none')
    edgecolor = kwargs.pop('edgecolor', 'k')

    x, y = mu
    U, S, V = np.linalg.svd(cov)
    theta = np.degrees(np.arctan2(U[1, 0], U[0, 0]))
    ellipsePlot = Ellipse(xy=[x, y],
                          width=2 * np.sqrt(S[0]) * factor,
                          height=2 * np.sqrt(S[1]) * factor,
                          angle=theta,
                          facecolor=facecolor, edgecolor=edgecolor, **kwargs)

    if ax is None:
        ax = pl.gca()
    ax.add_patch(ellipsePlot)

    return ellipsePlot


def hist2d(x, y, *args, **kwargs):
    """
    Plot a 2-D histogram of samples.

    """
    ax = kwargs.pop("ax", pl.gca())

    extent = kwargs.pop("extent", [[x.min(), x.max()], [y.min(), y.max()]])
    bins = kwargs.pop("bins", 50)
    color = kwargs.pop("color", "k")
    linewidths = kwargs.pop("linewidths", None)
    plot_datapoints = kwargs.get("plot_datapoints", True)
    plot_contours = kwargs.get("plot_contours", True)

    cmap = cm.get_cmap("gray")
    cmap._init()
    cmap._lut[:-3, :-1] = 0.
    cmap._lut[:-3, -1] = np.linspace(1, 0, cmap.N)

    X = np.linspace(extent[0][0], extent[0][1], bins + 1)
    Y = np.linspace(extent[1][0], extent[1][1], bins + 1)
    try:
        H, X, Y = np.histogram2d(x.flatten(), y.flatten(), bins=(X, Y),
                                 weights=kwargs.get('weights', None))
    except ValueError:
        raise ValueError("It looks like at least one of your sample columns "
                         "have no dynamic range. You could try using the "
                         "`extent` argument.")

    V = 1.0 - np.exp(-0.5 * np.arange(0.5, 2.1, 0.5) ** 2)
    Hflat = H.flatten()
    inds = np.argsort(Hflat)[::-1]
    Hflat = Hflat[inds]
    sm = np.cumsum(Hflat)
    sm /= sm[-1]

    for i, v0 in enumerate(V):
        try:
            V[i] = Hflat[sm <= v0][-1]
        except:
            V[i] = Hflat[0]

    X1, Y1 = 0.5 * (X[1:] + X[:-1]), 0.5 * (Y[1:] + Y[:-1])
    X, Y = X[:-1], Y[:-1]

    if plot_datapoints:
        ax.plot(x, y, "o", color=color, ms=1.5, zorder=-1, alpha=0.1,
                rasterized=True)
        if plot_contours:
            ax.contourf(X1, Y1, H.T, [V[-1], H.max()],
                        cmap=LinearSegmentedColormap.from_list("cmap",
                                                               ([1] * 3,
                                                                [1] * 3),
                        N=2), antialiased=False)

    if plot_contours:
        ax.pcolor(X, Y, H.max() - H.T, cmap=cmap)
        ax.contour(X1, Y1, H.T, V, colors=color, linewidths=linewidths)

    data = np.vstack([x, y])
    mu = np.mean(data, axis=1)
    cov = np.cov(data)
    if kwargs.pop("plot_ellipse", False):
        error_ellipse(mu, cov, ax=ax, edgecolor="r", ls="dashed")

    ax.set_xlim(extent[0])
    ax.set_ylim(extent[1])
    







def hist_1(xs, labels=None, lims=None, truths=None,nbins=[50],colin ='b',truth_color = 'r',annotate = None, figname = 'histplot.pdf'):
 
 fig = pl.figure()
 ax1 = fig.add_subplot(111)
 
 if (labels != None):
  ax1.set_xlabel(labels[0])
  
 
 if (lims != None):
  ax1.set_xlim(lims[0])
 
 
 if (len(nbins) > 0):
  bins = np.array(nbins)
 else:
  bins = nbins[0]
 
 if (truths != None):
  
  ylimnew = ax1.get_ylim
  ax1.vlines(truths[0],ylimnew[0],ylimnew[1],col = truth_color)
  
 ax1.hist(xs,bins = nbins[0], histtype='step',normed=1,color = colin)
 
 if (annotate != None):
  ax1.text(xtxtcoord,ytxtcoord-0.05,annotate,ha='right',transform=ax1.transAxes)
 
 pl.savefig(figname)
 
 return()
  
 
 
def corner(xs, weights=None, labels=None, extents=None, truths=None,truth_color="#4682b4", scale_hist=False, quantiles=[],verbose=True, plot_contours=False, plot_datapoints=False,fig=None,sigconts=[100.-68,100.-95,100-99.7],figname = 'histplot.pdf',annotate=[], **kwargs):


 
 ndim = np.shape(xs)
 
 if (ndim[1] == 1):
  if (len(annotate) > 0):
   text = annotate[0][0]
  hist_1(xs, labels=labels, truths=truths,colin = 'k',figname = figname, truth_color = truth_color, annotate = text)
  
 else:
  corner_1(xs, weights=weights, labels=labels, extents=extents, truths=truths,
           truth_color=truth_color, scale_hist=scale_hist, quantiles=quantiles,
           verbose=verbose, plot_contours=plot_contours, plot_datapoints=plot_datapoints,
           fig=fig,sigconts=sigconts,figname = figname, annotate = annotate, **kwargs)

 return()
  


import numpy as np
import matplotlib.pylab as plt
import scipy.optimize as mcfit






#generate a test polynomial function and add errors
ain = 2.
bin = 5.3
cin = 10.
x    = np.arange(100)
y    = ain*x**2 + bin*x + cin
sigy = np.ones(100)
y    = y + np.random.randn(100)*sigy #error of sigy about mean 0+ 0.



#define polynoimal function and fit
def func(x, a, b ,c):
 return(a*x**2 + b*x + c)
 

#fit function
popt, pcov = mcfit.curve_fit(func, x, y, sigma=sigy)
 


#output
xres = np.arange(0,100,0.1) 
yres = popt[0]*xres**2 + popt[1]*xres + popt[2] 

#sample probability space to generate errors






#plot the result
fig = plt.figure()
ax1 = fig.add_subplot(111)
ax1.plot(xres,yres)
ax1.errorbar(x,y,sigy,ls='')
plt.show()








import numpy as num

def DipTestSig(xpdf,nboot):
    """Dip test with significance
    """

    (dip,xlow,xup,ifault,gcm,lcm,mn,mj)=DipTest(xpdf)

    bootDip=[]
    for i in range(nboot):
        unifpdf=num.sort(num.random.rand(xpdf.shape[0]))
        bootDip.append(DipTest(unifpdf)[0])
    bootDip=num.array(bootDip)
    p=num.sum(num.less(dip,bootDip))/float(nboot)

    return (dip,p,xlow,xup)

def DipTest(xpdf):
    """Hartigan's dip test. 
    This is a copy 
    """
    x=num.sort(xpdf)
    N=x.shape[0]
    mn=num.zeros(x.shape,dtype=x.dtype)
    mj=num.zeros(x.shape,dtype=x.dtype)
    lcm=num.zeros(x.shape,dtype=x.dtype)
    gcm=num.zeros(x.shape)
    ifault=False
    
    #Check that N is positive
    if N<=0:
        ifault=1
        print '\nHartigansDipTest.    InputError :  ifault=%s'%str(ifault)
        return (dip,xl,xu,ifault,gcm,lcm,mn,mj)
    
    #check if N is one
    if N==1:
        xl=x[0]
        xu=x[0]
        dip=0.0
        ifault=2
        print '\nHartigansDipTest.    InputError :  ifault=%s'%str(ifault)
        return (dip,xl,xu,ifault,gcm,lcm,mn,mj)
    
    #check for case 1<N<4 or all identical values
    if N<=4 or x[N-1]==x[0]:
        xl=x[0]
        xu=x[0]
        dip=0.0
        ifault=4
        print '\nHartigansDipTest.    InputError :  ifault=%s'%str(ifault)
        return (dip,xl,xu,ifault,gcm,lcm,mn,mj)
    
    #check if x is perfectly unimodal
    xsign=-num.sign(num.diff(num.diff(xpdf)))
    posi=num.greater(xsign,0.0)
    negi=num.less(xsign,0.0)
    if num.sum(posi)==0 or num.sum(negi)==0 or num.sum(num.less(posi,num.min(negi)))==N:
        #A unimodal function is its own best unimodal approximation, 
        #with a zero corresponding dip
        xl=x[0]
        xu=x[N-1]
        dip=0.0
        ifault=5
        print '\nHartigansDipTest.    InputError :  ifault=%s'%str(ifault)
        return (dip,xl,xu,ifault,gcm,lcm,mn,mj)

    # LOW  contains the index of the current estimate of the lower 
    # end of the modal interval
    # HIGH contains the index of the current estimate of the upper 
    # end of the modal interval
    fn=N
    low=1
    high=N    
    dip=1./fn
    xl=x[low -1]
    xu=x[high -1]

    # establish the indices over which combination is necessary 
    # for the convex minorant fit
    mn[0]=1.
    for j in range(2,N+1):
        mn[j-1]=j-1
        
        mnj=mn[j-1]
        mnmnj=mn[mnj-1]
        a=mnj-mnmnj
        b=j-mnj
        while not ((mnj==1) or (x[j-1]-x[mnj-1])*a < (x[mnj-1]-x[mnmnj-1])*b):
            mn[j-1]=mnmnj
            mnj=mn[j-1]
            mnmnj=mn[mnj-1]
            a=mnj-mnmnj
            b=j-mnj

    # establish the indices over which combination is necessary 
    # for the concave majorant fit
    mj[N-1]=N
    na=N-1
    for jk in range(1,na+1):
        k=N-jk
        mj[k-1]=k+1
        
        mjk=mj[k-1]
        mjmjk=mj[mjk-1]
        a=mjk-mjmjk
        b=k-mjk
        while not ( (mjk==N) or (x[k-1]-x[mjk-1])*a<(x[mjk-1]-x[mjmjk-1])*b):
            mj[k-1]=mjmjk
            mjk=mj[k-1]
            mjmjk=mj[mjk-1]
            a=mjk-mjmjk
            b=k-mjk

    itarate_flag=True

    while itarate_flag:
        ic=1
        gcm[0]=high
        igcm1=gcm[ic-1]
        ic+=1
        gcm[ic-1]=mn[igcm1-1]
        
        while gcm[ic-1]>low:
            igcm1=gcm[ic-1]
            ic+=1
            gcm[ic-1]=mn[igcm1-1]

        icx=ic

        # collect the change points for the LCM from LOW to HIGH
        ic=1
        lcm[0]=low
        lcm1=lcm[ic-1]
        ic+=1
        lcm[ic-1]=mj[lcm1-1]
        while lcm[ic-1]<high:
            lcm1=lcm[ic-1]
            ic+=1
            lcm[ic-1]=mj[lcm1-1]
            
        icv=ic

        # ICX, IX, IG are counters for the convex minorant
        # ICV, IV, IH are counters for the concave majorant
        ig=icx
        ih=icv
        
        # find the largest distance greater than 'DIP' 
        # between the GCM and the LCM from low to high
        
        ix=icx-1
        iv=2
        d=0.0
        
        if not (icx<>2 or icv<>2):
            d=1./fn
        else:
            iterate_BP50=True
            
            while iterate_BP50:
                igcmx=gcm[ix-1]
                lcmiv=lcm[iv-1]
                if not (igcmx > lcmiv):
                    # if the next point of either the GCM or 
                    # LCM is from the LCM then calculate distance 
                    #here OTHERWISE, GOTO BREAK POINT 55
                    
                    lcmiv1=lcm[iv-1-1]
                    a=lcmiv-lcmiv1
                    b=igcmx-lcmiv1-1
                    dx=(x[igcmx-1]-x[lcmiv1-1])*a/(fn*(x[lcmiv-1]-x[lcmiv1-1]))-b/fn
                    ix-=1
                    if dx<d:
                        goto60=True
                    else:
                        d=dx
                        ig=ix+1
                        ih=iv
                        goto60=True
                else:
                    # if the next point of either the GCM or 
                    # LCM is from the GCM then calculate distance 
                    # here CODE BREAK POINT 55
                    lcmiv=lcm[iv-1]
                    igcm=gcm[ix-1]
                    igcm1=gcm[ix+1-1]
                    a=lcmiv-igcm1+1
                    b=igcm-igcm1
                    dx=a/fn - ((x[lcmiv-1]-x[igcm1-1])*b)/(fn*(x[igcm-1]-x[igcm1-1]))
                    iv+=1
                
                    if not dx<d:
                        d=dx
                        ig=ix+1
                        ih=iv-1
                        
                    goto60=True

                if goto60:
                    if ix<1 : ix=1
                    if iv>icv : iv=icv
                    iterate_BP50 = gcm[ix-1] <> lcm[iv-1]

        itarate_flag= not d<dip
        if itarate_flag:
            # if itarate_flag is true, then continue 
            # calculations and the great iteration cycle
            #if itarate_flag is NOT true, then stop 
            # calculations here, and break out of 
            #great iteration cycle to BREAK POINT 100
   
            # calculate the DIPs for the current LOW and HIGH

            #the DIP for the convex minorant
            dl=0.
            if ig <>icx:
                icxa=icx-1
                for j in range(ig,icxa+1):
                    temp=1./fn
                    jb=gcm[j+1-1]
                    je=gcm[j-1]
                    if not (je-jb<=1):
                        if not (x[je-1]==x[jb-1]):
                            a=je-jb
                            const=a/(fn*(x[je-1]-x[jb-1]))
                            for jr in range(int(jb),int(je+1)):
                                b=jr-jb+1
                                t=b/fn-(x[jr-1]-x[jb-1])*const
                                if (t>temp): temp=t
                    if dl<temp: dl=temp

            du=0.
            if not(ih==icv):
                icva=icv-1
                for k in range(ih,icva+1):
                    temp=1./fn
                    kb=lcm[k-1]
                    ke=lcm[k+1-1]
                    if not (ke-kb<=1):
                        if not (x[ke-1]==x[kb-1]):
                            a=ke-kb
                            const=a/(fn*(x[ke-1]-x[kb-1]))
                            for kr in range(int(kb),int(ke+1)):
                                b=kr-kb-1
                                t=(x[kr-1]-x[kb-1])*const-b/fn
                                if t>temp: temp=t
                    if du<temp: du=temp

            dipnew=dl
            if du>dl: dipnew=du
            if dip<dipnew: dip=dipnew
            low=gcm[ig-1]
            high=lcm[ih-1]
        #end if itarate_flag

    dip*=0.5
    xl=x[low-1]
    xu=x[high-1]
    
    return (dip,xl,xu,ifault,gcm,lcm,mn,mj)


if __name__=="__main__":
    """
    xpdf=num.array([-0.316502,  
           -0.4760215,  
           -0.2745295,  
           -0.043012,  
           -0.439517,  
           0.033979,  
           0.1215105,  
           0.0189845,  
           -0.246039,  
           0.1554785,  
           -0.333038,  
           -0.3330585,  
           0.095484,  
           0.1445005,  
           -0.2930415,  
           -0.192532,  
           -0.011058,  
           -0.035037,  
           -0.3665275,  
           0.084498,  
           0.012455,  
           -0.3340235,  
           -0.2225445,  
           -0.1030285,  
           -0.161511,  
           -0.257043,  
           -0.257026,  
           -0.251022,  
           -0.2730175,  
           -0.017697,  
           0.00498,  
           -0.252074,  
           -0.1900265,  
           -0.288551,  
           -0.1440105,  
           -0.273011,  
           -0.30153,  
           -0.3870235,  
           -0.302027,  
           -0.2765615,  
           -0.3289815,  
           0.0689785,  
           -0.02304,  
           -0.2210335,  
           -0.175028,  
           -0.165512,  
           -0.077005,  
           -0.270514,  
           -0.3465375,  
           -0.242042,  
           0.298475,  
           -0.2780885,  
           0.063467,  
           -0.34901  ])
           """
    xpdf=num.random.randn(100)
    xpdf+=num.random.randn(100)+5
    print DipTestSig(xpdf,1000)
#print DipTest(xpdf)


#code to compare the model and lag estiamtes for cream fits with different
#error bar treatments for a few example targets
import numpy as np
import matplotlib.pylab as plt
import matplotlib.gridspec as gridspec
import glob
import astropy_stark.cream_posterior as cpos





def lcplot(dnow,title='',idburnin=2./3,justth=0,justcont=0,plotinfo=1,
           plottrace=0,plots_per_page=5,xlclab = 'Time (HJD - 50,000)',xtflab ='lag (days)',forcelab=[],forcelag=[],sameplotdrive=1,extents=[],justnewsig=0,taumeanplot=1,tau90plot=0,postplot=1,header='',tauplot0=0,gplot=1,true=['','',np.log10(0.75)]):
 '''
 #input dirres (list of target directories from which to make plot (1plot per list element))
 #... if no ../output_20xxxx at end of each string, cream will look for these automatically and use the most
 #... recent simulation in te directory
 #tit... list of plot titles defaults to lcplot_x.pdf where x is the integer of list element
 plot the results of a cream simulation
 #if justth = 1 then only plot lines
 #if justcont = 1 then only plot continuum
 #if fitinfo = 1 then plot the parameter trace plots
 #if plottrace = 1 then plot the trace plots of the parameters
 #if plotsperpage == -1 then have one page per wavelength
 :param dnow:
 :param title:
 :param idburnin:
 :param justth:
 :param justcont:
 :param plotinfo:
 :param plottrace:
 :param plots_per_page:
 :param xlclab:
 :param xtflab:
 :param forcelab:
 :param forcelag:
 :param sameplotdrive:
 :param extents:
 :param justnewsig:
 :param taumeanplot:
 :param tau90plot:
 :param postplot:
 :param header:
 :param tauplot0:
 :param gplot:
 :param true:
 :return:
 '''
 taumean = 1
 idcount = 1
 dirres = []
 tit = []
 head = []
 if 'output_20' in dnow:
  dirres.append(dnow)
 else:
  a = sorted(glob.glob(dnow+'/output_20*'))[-1]
  dirres.append(a)
 if (title == ''):
  tit.append('lcplot_'+np.str(idcount))
 elif isinstance(title,str):
  tit.append(title+'_'+np.str(idcount))
 else:
  tit = list(title)
 head.append(header)

 '''
 load data from relevant files
 '''
 print('cream_lcplot plotting results from...',dnow)
 fileth = '/outputpars_th.dat'
 filetf = '/plots/modeltf.dat'
 filetfsig = '/plots/modeltf_sig.dat'
 filemod = '/plots/modellc.dat'
 filemodsig = '/plots/modellc_sig.dat'
 filevarexp = '/outputpars_varexpand.dat'
 filesigexp = '/outputpars2.dat'
 filedrive = '/plots/modeldrive.dat'
 filebof= '/testbofs.dat'
 filepspec = '/cream_furparms.dat'










 ndres = len(dirres)
 for idnow in range(ndres):


  #put this in a loop to make plots for all tested targets
  dnow = dirres[idnow]

  #read cream names file
  f = open(dnow+'/../creamnames.dat')
  filedat = f.readlines()
  filedat = [i.strip() for i in filedat]#remove \n
  wav     = [i.split(' ')[-1] for i in filedat]
  wav = [float(x) for x in wav]
  filedat = [i.split(' ')[0] for i in filedat]
  filedat = [i[1:-1] for i in filedat]
  f.close()
  nwav = len(wav)





  #!!!!!!!!!!!!!!!!!!!!!! load the model (all wavelengths in one file) !!!!!!!!!
  try:
   mod = np.loadtxt(dnow+'/'+filemod)
  except:
   mod = np.loadtxt(dnow+filemod)

  idxinc = np.where(mod[:,1] != 0)[0]
  tlo, thi = mod[idxinc[0],0],mod[idxinc[-1],0]
  mod = mod[idxinc,:]
  sigmod = np.loadtxt(dnow+'/'+filemodsig)
  tmod = mod[:,0]
  mod  = mod[:,1:]
  sigmod = sigmod[idxinc,1:]
  #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!




  #!!!!!!!!!!!!!!!!!!!!! load the response function !!!!!!!!!!!!!!!!!!!!!!!!!!!
  modtf = np.loadtxt(dnow+'/'+filetf)
  sigtf = np.loadtxt(dnow+'/'+filetfsig)
  tau   = modtf[:,0]
  modtf = modtf[:,1:]
  sigtf = sigtf[:,1:]

  #try to load top hat parameters, replace the disk response if non zero
  modth = np.loadtxt(dnow+'/'+fileth)[:,:nwav]
  nits = np.shape(modth[:,0])[0]
  modth = modth[int(idburnin*nits):,:]
  thstats = np.percentile(modth,[15.865,50,84.135],axis=0)
  thmed = thstats[1,:]
  idxth  = np.where(thmed != 0)[0]
  #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!






  #!!!!!!!!! load the error bar modification parameters calculate the median parameter and apply it
  try:
   sigexp = np.loadtxt(dnow+'/'+filesigexp)[:,2*nwav:]
   nits = np.shape(sigexp[:,0])[0]
   sigexp = sigexp[int(idburnin*nits):,:]
   semed = np.median(sigexp,axis=0)
  except:
   print('no error bar f factors')
   semed = np.ones(nwav)

  try:
   varexp = np.loadtxt(dnow+'/'+filevarexp)
   nits = np.shape(varexp[:,0])[0]
   varexp = varexp[int(idburnin*nits):,:]
   vamed = np.median(varexp,axis=0)
  except:
   print('no var expand parameters')
   vamed = np.zeros(nwav)

  try:
   #data required for outlier rejection
   fileoutrej = [dnow+'/plots/data_echo_dat_'+filedat[ilc] for ilc in range(len(wav))]#sorted(glob.glob(dnow+'/plots/data_echo_dat_*'))
   drej = [np.loadtxt(fileoutrej[idx]) for idx in range(nwav)]
  except:
   print('no outlier rejection found in...',dnow+'/plots/data_echo_dat_*')
   drej = [np.zeros((1,4)) for idx in range(nwav)]
















  #!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#
  #!!!!!!!!!!!!!!!# trace plots !!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!
  #!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#
  if (plottrace == 1):
   filetrace = ['/outputpars.dat','/outputpars2.dat','/outputpars_th.dat','/outputpars_varexpand.dat']
   idtrace = [[2,3,4],list(np.arange(3*nwav)),list(np.arange(2*nwav)),list(np.arange(nwav))]
   tit_trace = [[r'$\log \dot{M}$',r'$\cos i$',r'$\alpha$'],
   ['stretch '+np.str(idx) for idx in range(nwav)] + ['offset '+np.str(idx) for idx in range(nwav)] + ['sig f '+np.str(idx) for idx in range(nwav)],
   [r'$\tau_\mathrm{cent}$ '+np.str(idx) for idx in range(nwav)] +  [r'$\tau_\mathrm{width}$ '+np.str(idx) for idx in range(nwav)],
   ['sig_V '+np.str(idx) for idx in range(nwav)]
   ]

   nft = len(filetrace)
   idtot  = 0
   ptraceop = []
   ttraceop = []
   for idx in range(nft):
    npar = len(idtrace[idx])
    try:
     dtrace = np.loadtxt(dnow+filetrace[idx])[:,idtrace[idx]]
     vartrace = np.std(dtrace,axis=0)
     idinc = np.where(vartrace > 0)[0]
     d_trace = [dtrace[:,idx2] for idx2 in idinc]
     t_trace = [tit_trace[idx][id2] for id2 in range(npar) if id2 in idinc]
     ptraceop.append(d_trace)


     ttraceop.append(t_trace)
    except:
     print('cannot find file for trace plots in...')
     print(dnow+filetrace[idx])

   ptrace_plot = [j for i in ptraceop for j in i]
   ttrace_plot = [j for i in ttraceop for j in i]

   ntrace = len(ptrace_plot)
   nalong = min(4,ntrace)


   ndown = np.int(np.ceil(1.*ntrace/nalong))
   gs1 = gridspec.GridSpec(ndown, nalong)
   gs1.update(left=0.1, right=0.9, wspace=0.05,hspace = 0.0,bottom=0.1,top=0.99)

   iynow = -1
   for idx in range(ntrace):
    ixnow = np.mod(idx,nalong)
    if (ixnow == 0):
     iynow = iynow + 1
    ax1 = plt.subplot(gs1[iynow, ixnow])
    ax1.plot(ptrace_plot[idx])
    ax1.set_ylabel(ttrace_plot[idx])
    print('traceplot...',iynow,ixnow,np.mean(ptrace_plot[idx]), np.std(ptrace_plot[idx]))
   #plt.tight_layout()
   ax1.set_title(head[idnow])
   plt.savefig('traceplot_'+np.str(tit[idnow])+'.pdf')





  #!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#
  #!!!!!!!!!!!!!!!# driver, power spectrum and BOF plots !!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!
  #!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!#
  boffile = np.loadtxt(dnow + filebof)
  cisq = boffile[:,0]

  if (plotinfo == 1):
   boffur = boffile[:,1]
   boftot = cisq + boffur

   pspecfile = np.loadtxt(dnow + filepspec,skiprows=1)
   freq = pspecfile[1:,0]/2/np.pi
   pspec = pspecfile[1:,[1,2]]

   datdrive = np.loadtxt(dnow + filedrive)



   ndown = 2
   nalong = 2
   gs1 = gridspec.GridSpec(ndown, nalong)
   gs1.update(left=0.1, right=0.9, wspace=0.3,hspace = 0.3,bottom=0.1,top=0.99)

   ax1 = plt.subplot(gs1[0, :])
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1],color='k',linewidth=0.5)
   ax1.fill_between(datdrive[idxinc,0],datdrive[idxinc,1]-datdrive[idxinc,2],datdrive[idxinc,1]+datdrive[idxinc,2],alpha = 0.3,color='k')
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]-datdrive[idxinc,2],color='k')
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]+datdrive[idxinc,2],color='k')
   ax1.set_xlabel('time (days)')
   ax1.set_ylabel('X(t) (arbitrary units)')

   ax1 = plt.subplot(gs1[1,0])
   ax1.plot(boftot,label=r'$\chi^2+\mathrm{BOF}_\mathrm{fur}$' )
   ax1.plot(cisq,label=r'$\chi^2$')
   ax1.plot(boffur,label=r'$\mathrm{BOF}_\mathrm{fur}$')
   ax1.set_xlabel('Iteration')
   ymin = np.min(boffur)
   ymax = np.mean(boftot) + np.std(boftot)*2#ymin + np.max(
   ylim = [ymin,ymax]
   ax1.set_ylim(ylim)
   ax1.set_ylabel('BOF')
   plt.legend(fontsize='small')
   ax1 = plt.subplot(gs1[1,1])
   ps = pspec[:,0]**2 + pspec[:,1]**2
   ax1.scatter(freq,ps)
   try:
    sigpspec = pspecfile[1:,[3,4]]
    ps_sd = 2* np.sqrt( (pspec[:,0]*sigpspec[:,0])**2 + (sigpspec[:,1]*sigpspec[:,1])**2 )
    ax1.errorbar(freq,ps,ps_sd)
    #ax1.fill_between(freq,ps,ps-ps_sd,ps+ps_sd)
   except:
    print('cannot find uncertanties for power spectrum plot')

   xlim = [freq[0],freq[-1]]
   ax1.set_xlim(xlim)
   ax1.set_xscale('log')
   ax1.set_yscale('log')
   ax1.set_xlabel('frequency (cycles/day)')
   ax1.set_ylabel('P(f)')
   ax1.set_title(head[idnow])

   plt.savefig('fitinfo_'+np.str(tit[idnow])+'.pdf')















  #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!





  nwavplot = nwav
  if (justcont == 1):
   idth = np.where(np.array(wav) == -1)[0]
   nth = np.shape(idth)[0]
   nwavplot = nwav - nth

  if (justth == 1):
   idcont = np.where(np.array(wav) != -1)[0]
   ncont = np.shape(idcont)[0]
   nwavplot = nwav - ncont

  ####!!!!       initialise the figure
  #grid spec gives better customisation of plot layout. Copied and pasted from
  #pyplot_cream_easy_2.py in mcmcmulti3 directory other codes below customise for this case






  iwavinc = [i for i in range(nwav)]
  if (justth == 1):
   iwavinc = [i for i in range(nwav) if wav[i] == -1]
  if (justcont == 1):
   iwavinc = [i for i in range(nwav) if wav[i] != -1]
  nwavinc = len(iwavinc)

  if (plots_per_page == -1):
   wavu = np.sort(np.unique(wav))
   npages = len(wavu)
  else:
   npages   = np.int(np.ceil(1.0*nwavinc/plots_per_page))


  #organise how to distribute the plots
  if (plots_per_page == -1):
   iwavinc_page = []
   for iw2 in range(npages):
    wavunow = wavu[iw2]
    iwnow = [iwavinc[i2] for i2 in range(nwavinc) if wav[iwavinc[i2]] == wavunow ]
    iwavinc_page.append(iwnow)

  elif (nwavinc > plots_per_page):
   iwavinc_page = [iwavinc[i:i+npages] for i  in range(0, len(iwavinc), npages)]

  else:
   iwavinc_page = [iwavinc]



  for ipage in range(npages):
   nwavplot = len(iwavinc_page[ipage])

   if (sameplotdrive == 1 and ipage == 0):
    ipnow = 1
    gs1 = gridspec.GridSpec(nwavplot+1, 4)
    nshapegs = nwavplot+1
   else:
    ipnow = 0
    gs1 = gridspec.GridSpec(nwavplot, 4)
    nshapegs = nwavplot


   gs1.update(left=0.1, right=0.9, wspace=0.05,hspace = 0.0,bottom=0.15,top=0.99)
   ###!!!!!


   #if ipage == 0 then plot the driver up top
   if (sameplotdrive == 1 and ipage == 0):
    ax1 = plt.subplot(gs1[0, 1:])
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1],color='k',linewidth=0.5)
    ax1.fill_between(datdrive[idxinc,0],datdrive[idxinc,1]-datdrive[idxinc,2],datdrive[idxinc,1]+datdrive[idxinc,2],alpha = 0.3,color='k')
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]-datdrive[idxinc,2],color='k')
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]+datdrive[idxinc,2],color='k')
    ax1.set_xticklabels([])
    ax1.tick_params(direction='in')
    ax1.set_yticks([])
    ax1.set_xlim([tlo,thi])


   for ilc in iwavinc_page[ipage]:

    ax1 = plt.subplot(gs1[ipnow, 1:])
    axtf = plt.subplot(gs1[ipnow,:1])

    dat = np.loadtxt(dnow+'/../'+filedat[ilc])
    tdat = dat[:,0]
    xdat = dat[:,1]
    sigdat = dat[:,2]
    sigdat_new = np.sqrt((semed[ilc]*sigdat)**2 + vamed[ilc])
    xm = mod[:,ilc]
    sm = sigmod[:,ilc]

    #decide order of old/new error bar plotting
    for it in range(np.shape(xdat)[0]):
     signew = sigdat_new[it]
     sigold = sigdat[it]
     if (signew > sigold):
      ax1.errorbar([tdat[it]],[xdat[it]],[signew],ls='',color='b')
      if (justnewsig == 0):
       ax1.errorbar([tdat[it]],[xdat[it]],[sigold],ls='',color='r')
     else:
      if (justnewsig == 0):
       ax1.errorbar([tdat[it]],[xdat[it]],[sigold],ls='',color='r')
      ax1.errorbar([tdat[it]],[xdat[it]],[signew],ls='',color='b')



    #plot outlier rejected points if any
    drnow = drej[ilc]
    idrej = np.where(drnow[:,3] > 0)[0]
    ax1.scatter(drnow[idrej,0],drnow[idrej,1],color='r',marker='o')

    print(tmod[10],xm[10:15],ilc)
    ax1.plot(tmod,xm)
    ax1.fill_between(tmod,xm-sm,xm+sm,alpha=0.2)

    #set ylim by mod or data
    ymodlo = np.min(xdat[:]-sigdat[:])
    ymodhi = np.max(xdat[:]+sigdat[:])
    yrange = ymodhi - ymodlo
    ylim = [ymodlo-yrange/10,ymodhi+yrange/10]
    ax1.set_ylim(ylim)


    #check of we have top hat or disk response function
    if (ilc in idxth):
     axtf.hist(modth[:,ilc],bins=tau)
     thlags = np.percentile(modth[:,ilc],[15.865,50,84.135])
     lo = np.str(np.round(thlags[1]-thlags[0],2))
     med = np.str(np.round(thlags[1],2))
     hi = np.str(np.round(thlags[2] - thlags[1],2))
     lagtxt=r'$\tau_\mathrm{cent}='+med+'^{+'+hi+'}_{-'+lo+'}$'

     tfyl = list(axtf.get_ylim())
     axtf.set_ylim(tfyl)

     if (taumean == 1):
      axtf.text(0.9,0.8,lagtxt,ha='right',transform=axtf.transAxes,fontsize=8)
      axtf.plot([thlags[0]]*2,tfyl,ls=':',label=None,color='k')
      axtf.plot([thlags[1]]*2,tfyl,ls='-',label=None,color='k')
      axtf.plot([thlags[2]]*2,tfyl,ls=':',label=None,color='k')
    else:
     #disk response function
     #response function only saved once in wavelength file
     #if have multiple lc's at same wavelength, need to plot the response function
     #from the first light curve at that wavelength
     wavnow = wav[ilc]
     idsamewav = np.where(np.array(wav) == wavnow)[0]
     if (np.shape(idsamewav)[0] > 0):
      itf = idsamewav[0]
     else:
      itf = ilc
     axtf.plot(tau,modtf[:,itf])
     axtf.fill_between(tau,modtf[:,itf]-sigtf[:,itf],modtf[:,itf]+sigtf[:,itf],alpha=0.2)
     yltf = list(axtf.get_ylim())
     if (taumeanplot == 1):
      taumean = np.sum(tau*modtf[:,itf])/np.sum(modtf[:,itf])
      axtf.plot([taumean]*2,yltf,color='k')
     if (tau90plot > 0):
      Ntau = np.shape(tau)[0]
      psi90 = tau90plot
      psimax = np.max(modtf[:,itf])
      for itau in range(Ntau-1,0,-1):
       psinow = modtf[itau,itf]/psimax
       if (psinow > psi90):
        tau90 = tau[itau]
        id90 = itau
        break
      axtf.plot([tau[0],tau[-1]],[psi90]*2,color='r',ls='--')
      axtf.plot([tau90]*2,yltf,color='r',ls='--')
      if (tauplot0 == 1):
       axtf.plot([tau[0],tau[-1]],[0]*2,color='k')
    #miscellaneous label positions/tick mark info etc
    if (ipnow == nshapegs - 1):
     axtf.set_xlabel(xtflab)
     ax1.set_xlabel(xlclab)
    else:
     ax1.set_xticklabels([])
     axtf.set_xticklabels([])
    if (ipnow == nshapegs/2):
     axtf.set_ylabel('response (relative units)')
     ax1.set_ylabel('flux (relative units)')





    ax1.tick_params(direction='in')
    axtf.set_yticklabels([])
    axtf.set_yticks([])
    ax1.set_yticks([])
    ax1.yaxis.set_label_position('right')
    ax1.set_yticklabels([])

    if (forcelag != []):
     axtf.set_xlim(forcelag)

    if (forcelab == []):
     ax1.text(0.9,0.8,filedat[ilc],ha='right',transform=ax1.transAxes,fontsize=8)
    elif (forcelab == 0):
     pass
    else:
     ax1.text(0.9,0.8,forcelab[ilc],ha='right',transform=ax1.transAxes,fontsize=8)

    #title
    if (ipnow == 0):
     ax1.text(0.5,1.2,tit[idnow],ha='center',transform=ax1.transAxes,fontsize=14)

    ax1.set_xlim([tlo,thi])
    ipnow = ipnow + 1

   ax1.set_title(head[idnow])
   plt.savefig('page_'+np.str(np.int(ipage))+'_'+'lcplot_'+np.str(tit[idnow])+'.pdf')
   plt.close()


  if (postplot == 1):
   print('making posterior plot....','posterior_'+tit[idnow]+'.pdf')
   try:
    cpos.cream_posterior(dnow,true=true,header=head[idnow],extents_in=extents,fsave='posterior_'+tit[idnow]+'.pdf')
   except:
    print('unable to make covariance plot for disc posteriors. Please check at least some of these are set to vary'
          'in the fit.')




 #make the g plot
 if (gplot == 1):
  with open(dnow+'/cream_furparms.dat') as f:
   content = f.readlines()
   content = [x.strip() for x in content]
   f.close()
  x = [np.float(cnow) for cnow in content[0].split()]
  p0,dw,w0 = x
  dfur = np.loadtxt(dnow+'/cream_furparms.dat',skiprows=1)
  fnow,sfur,cfur,sd_sfur,sd_cfur = dfur.T
  sig0_2 = 0.5*p0*dw*((w0/2/np.pi)/fnow)**2
  sig_c2 = sd_cfur**2
  sig_s2 = sd_sfur**2
  Gsk = sig0_2 / ( sig0_2 + sig_s2 )
  Gck = sig0_2 / (sig0_2 + sig_c2)
  nfurtot = np.shape(Gck)[0]
  fig = plt.figure()
  ax2 = fig.add_subplot(111)
  ax2.plot(fnow/2/np.pi,Gsk,label='Sk',color='r')
  ax2.plot(fnow/2/np.pi,Gck,label='Ck',color='b')
  sum_gs = np.sum(Gsk)
  sum_gc = np.sum(Gck)
  sum_ave = (sum_gc + sum_gs)/2
  ax2.set_xlabel('frequency cycles/day')
  ax2.set_ylabel(r'$G=\sigma_0^2 / \left( \sigma_0^2 + \sigma^2 \right)$')
  ax2.set_title('G factor')
  ax2.text(0.98,0.55,r'$N_{\mathrm{eff}}=\sum_k \frac{G_{Sk} + G_{Ck}}{2} = '+np.str(np.int(sum_ave))+'$ of '+np.str(np.int(nfurtot)),ha='right',transform=ax2.transAxes,fontsize=14)
  ax2.set_xscale('log')
  ax2.set_yscale('log')
  plt.legend()
  print(dnow+'/G_plot.pdf')
  plt.tight_layout()
  plt.savefig(dnow+'/G_plot.pdf')


  #save the effective chi squared and calculate the effective number of Fourier (and non Fourier parameters)
  cisqmed = np.median(cisq[int(idburnin*nits):])

  Nth = np.shape(idxth)[0]
  fdisk = np.loadtxt(dnow+'/outputpars.dat')[:,[2,3,4]]
  stddisk = np.std(fdisk,axis=0)
  Ndisk = np.shape(np.where(stddisk != 0)[0])[0]
  print('Nth ',Nth,' Ndisk',Ndisk)
  Np_notfur = 2*nwav + Ndisk + Nth
  if (np.mean(semed) == 0):
   semed_on = 0
  else:
   semed_on = 1
   Np_notfur = Np_notfur + nwav
  if (np.mean(vamed) == 0):
   vamed_on = 0
  else:
   vamed_on = 1
   Np_notfur = Np_notfur + nwav

  Np_fur = np.int(sum_ave)
  cisqmed_reduce = cisqmed/(Np_fur+Np_notfur)

  f = open(dnow+'/G_info.txt','w')
  f.write('N_freq_tot	N_freq_eff	N_non_freq	cisq(med)	cisq(med)/(N_non_freq + N_freq_eff)')
  op = np.str(nfurtot)+' '+np.str(Np_fur)+' '+np.str(np.int(Np_notfur))+' '+np.str(cisqmed_reduce)
  f.write(op+'\n')
  f.close()

 return()
